@@ -51,7 +51,7 @@ typedef struct {
     void *context;
 
     struct sockaddr_storage sendAddr;
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
     size_t sendAddrLength;
 #else
     socklen_t sendAddrLength;
@@ -65,7 +65,7 @@ typedef enum {
 } MultiCastType;
 
 typedef union {
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
     struct ip_mreq ipv4;
 #else
     struct ip_mreqn ipv4;
@@ -97,7 +97,7 @@ multiCastType(struct addrinfo *info) {
     return MULTICASTTYPE_NONE;
 }
 
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
 
 #define ADDR_BUFFER_SIZE 15000 /* recommended size in the MSVC docs */
 
@@ -230,7 +230,7 @@ setMulticastInterface(const char *netif, struct addrinfo *info,
     return UA_STATUSCODE_GOOD;
 }
 
-#endif /* _WIN32 */
+#endif /* UA_ARCHITECTURE_WIN32 */
 
 static UA_StatusCode
 setupMulticastRequest(UA_FD socket, MulticastRequest *req, const UA_KeyValueMap *params,
@@ -239,7 +239,7 @@ setupMulticastRequest(UA_FD socket, MulticastRequest *req, const UA_KeyValueMap
     if(info->ai_family == AF_INET) {
         struct sockaddr_in *sin = (struct sockaddr_in *)info->ai_addr;
         req->ipv4.imr_multiaddr = sin->sin_addr;
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
         req->ipv4.imr_interface.s_addr = htonl(INADDR_ANY); /* default ANY */
 #else
         req->ipv4.imr_address.s_addr = htonl(INADDR_ANY); /* default ANY */
@@ -529,7 +529,7 @@ setupSendMultiCast(UA_FD fd, struct addrinfo *info, const UA_KeyValueMap *params
     int result = -1;
     if(info->ai_family == AF_INET && multiCastType == MULTICASTTYPE_IPV4) {
         result = setsockopt(fd, IPPROTO_IP, IP_MULTICAST_IF,
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
                             (const char *)&req.ipv4.imr_interface,
                             sizeof(struct in_addr));
 #else
@@ -538,7 +538,7 @@ setupSendMultiCast(UA_FD fd, struct addrinfo *info, const UA_KeyValueMap *params
 #if UA_IPV6
     } else if(info->ai_family == AF_INET6 && multiCastType == MULTICASTTYPE_IPV6) {
         result = setsockopt(fd, IPPROTO_IPV6, IPV6_MULTICAST_IF,
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
                             (const char *)&req.ipv6.ipv6mr_interface,
 #else
                             &req.ipv6.ipv6mr_interface,
@@ -591,12 +591,10 @@ UDP_close(UA_POSIXConnectionManager *pcm, UDP_FD *conn) {
     pcm->fdsSize--;
 
     /* Signal closing to the application */
-    UA_UNLOCK(&el->elMutex);
     conn->applicationCB(&pcm->cm, (uintptr_t)conn->rfd.fd,
                         conn->application, &conn->context,
                         UA_CONNECTIONSTATE_CLOSING,
                         &UA_KEYVALUEMAP_NULL, UA_BYTESTRING_NULL);
-    UA_LOCK(&el->elMutex);
 
     /* Close the socket */
     int ret = UA_close(conn->rfd.fd);
@@ -657,7 +655,7 @@ UDP_connectionSocketCallback(UA_POSIXConnectionManager *pcm, UDP_FD *conn,
 
     /* Receive */
     struct sockaddr_storage source;
-#ifndef _WIN32
+#ifndef UA_ARCHITECTURE_WIN32
     socklen_t sourceSize = (socklen_t)sizeof(struct sockaddr_storage);
     ssize_t ret = recvfrom(conn->rfd.fd, (char*)response.data, response.length,
                            MSG_DONTWAIT, (struct sockaddr*)&source, &sourceSize);
@@ -718,12 +716,10 @@ UDP_connectionSocketCallback(UA_POSIXConnectionManager *pcm, UDP_FD *conn,
                  sourceAddr, sourcePort);
 
     /* Callback to the application layer */
-    UA_UNLOCK(&el->elMutex);
     conn->applicationCB(&pcm->cm, (uintptr_t)conn->rfd.fd,
                         conn->application, &conn->context,
                         UA_CONNECTIONSTATE_ESTABLISHED,
                         &kvm, response);
-    UA_LOCK(&el->elMutex);
 }
 
 static UA_StatusCode
@@ -775,7 +771,7 @@ UDP_registerListenSocket(UA_POSIXConnectionManager *pcm, UA_UInt16 port,
     MultiCastType mc = multiCastType(info);
 
     /* Bind socket to the address */
-#ifdef _WIN32
+#ifdef UA_ARCHITECTURE_WIN32
     /* On windows we need to bind the socket to INADDR_ANY before registering
      * for the multicast group */
     int ret = -1;
@@ -876,12 +872,10 @@ UDP_registerListenSocket(UA_POSIXConnectionManager *pcm, UA_UInt16 port,
     pcm->fdsSize++;
 
     /* Register the listen socket in the application */
-    UA_UNLOCK(&el->elMutex);
     connectionCallback(&pcm->cm, (uintptr_t)newudpfd->rfd.fd,
                        application, &newudpfd->context,
                        UA_CONNECTIONSTATE_ESTABLISHED,
                        &UA_KEYVALUEMAP_NULL, UA_BYTESTRING_NULL);
-    UA_LOCK(&el->elMutex);
     return UA_STATUSCODE_GOOD;
 }
 
@@ -1177,11 +1171,9 @@ UDP_openSendConnection(UA_POSIXConnectionManager *pcm, const UA_KeyValueMap *par
 
     /* Signal the connection as opening. The connection fully opens in the next
      * iteration of the EventLoop */
-    UA_UNLOCK(&el->elMutex);
     connectionCallback(&pcm->cm, (uintptr_t)newSock, application,
                        &conn->context, UA_CONNECTIONSTATE_ESTABLISHED,
                        &UA_KEYVALUEMAP_NULL, UA_BYTESTRING_NULL);
-    UA_LOCK(&el->elMutex);
 
     return UA_STATUSCODE_GOOD;
 }
