@@ -100,7 +100,6 @@ asym_getLocalSignatureSize_sp_basic256(const Basic256_ChannelContext *cc) {
     return mbedtls_pk_rsa(cc->policyContext->localPrivateKey)->len;
 #else
     return mbedtls_rsa_get_len(mbedtls_pk_rsa(cc->policyContext->localPrivateKey));
-
 #endif
 }
 
@@ -117,12 +116,12 @@ asym_getRemoteSignatureSize_sp_basic256(const Basic256_ChannelContext *cc) {
 
 static size_t
 asym_getRemotePlainTextBlockSize_sp_basic256(const Basic256_ChannelContext *cc) {
+    if(cc == NULL)
+        return 0;
 #if MBEDTLS_VERSION_NUMBER >= 0x02060000 && MBEDTLS_VERSION_NUMBER < 0x03000000
     mbedtls_rsa_context *const rsaContext = mbedtls_pk_rsa(cc->remoteCertificate.pk);
     return rsaContext->len - UA_SECURITYPOLICY_BASIC256SHA1_RSAPADDING_LEN;
 #else
-    if(cc == NULL)
-        return 0;
     return mbedtls_rsa_get_len(mbedtls_pk_rsa(cc->remoteCertificate.pk)) -
         UA_SECURITYPOLICY_BASIC256SHA1_RSAPADDING_LEN;
 #endif
@@ -151,7 +150,7 @@ asym_decrypt_sp_basic256(Basic256_ChannelContext *cc,
     if(cc == NULL || data == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
     return mbedtls_decrypt_rsaOaep(&cc->policyContext->localPrivateKey,
-                                   &cc->policyContext->drbgContext, data);
+                                   &cc->policyContext->drbgContext, data, MBEDTLS_MD_SHA1);
 }
 
 static size_t
@@ -166,12 +165,12 @@ asym_getRemoteEncryptionKeyLength_sp_basic256(const Basic256_ChannelContext *cc)
 
 static size_t
 asym_getRemoteBlockSize_sp_basic256(const Basic256_ChannelContext *cc) {
+    if(cc == NULL)
+        return 0;
 #if MBEDTLS_VERSION_NUMBER >= 0x02060000 && MBEDTLS_VERSION_NUMBER < 0x03000000
     mbedtls_rsa_context *const rsaContext = mbedtls_pk_rsa(cc->remoteCertificate.pk);
     return rsaContext->len;
 #else
-    if(cc == NULL)
-        return 0;
     return mbedtls_rsa_get_len(mbedtls_pk_rsa(cc->remoteCertificate.pk));
 #endif
 }
@@ -214,7 +213,7 @@ sym_verify_sp_basic256(Basic256_ChannelContext *cc,
         return UA_STATUSCODE_BADSECURITYCHECKSFAILED;
 
     Basic256_PolicyContext *pc = cc->policyContext;
-    
+
     unsigned char mac[UA_SHA1_LENGTH];
     mbedtls_hmac(&pc->sha1MdContext, &cc->remoteSymSigningKey, message, mac);
 
@@ -365,13 +364,12 @@ parseRemoteCertificate_sp_basic256(Basic256_ChannelContext *cc,
     /* Check the key length */
 #if MBEDTLS_VERSION_NUMBER >= 0x02060000 && MBEDTLS_VERSION_NUMBER < 0x03000000
     mbedtls_rsa_context *rsaContext = mbedtls_pk_rsa(cc->remoteCertificate.pk);
-    if(rsaContext->len < UA_SECURITYPOLICY_BASIC256_MINASYMKEYLENGTH ||
-       rsaContext->len > UA_SECURITYPOLICY_BASIC256_MAXASYMKEYLENGTH)
+    size_t keylen = rsaContext->len;
 #else
     size_t keylen = mbedtls_rsa_get_len(mbedtls_pk_rsa(cc->remoteCertificate.pk));
+#endif
     if(keylen < UA_SECURITYPOLICY_BASIC256_MINASYMKEYLENGTH ||
        keylen > UA_SECURITYPOLICY_BASIC256_MAXASYMKEYLENGTH)
-#endif
         return UA_STATUSCODE_BADCERTIFICATEUSENOTALLOWED;
 
     return UA_STATUSCODE_GOOD;
