@@ -32,25 +32,25 @@
 typedef struct {
     EVP_PKEY *localPrivateKey;
     UA_ByteString localCertThumbprint;
-    const UA_Logger *logger;    
+    const UA_Logger *logger;
 } Policy_Context_Basic256Sha256;
 
 typedef struct {
-    UA_ByteString localSymSigningKey;  
-    UA_ByteString localSymEncryptingKey; 
-    UA_ByteString localSymIv; 
+    UA_ByteString localSymSigningKey;
+    UA_ByteString localSymEncryptingKey;
+    UA_ByteString localSymIv;
     UA_ByteString remoteSymSigningKey;
     UA_ByteString remoteSymEncryptingKey;
     UA_ByteString remoteSymIv;
 
     Policy_Context_Basic256Sha256 *policyContext;
     UA_ByteString remoteCertificate;
-    X509 *remoteCertificateX509; /* X509 */      
+    X509 *remoteCertificateX509; /* X509 */
 } Channel_Context_Basic256Sha256;
 
 /* create the policy context */
 
-static UA_StatusCode 
+static UA_StatusCode
 UA_Policy_New_Context(UA_SecurityPolicy * securityPolicy,
                        const UA_ByteString localPrivateKey,
                        const UA_Logger *logger) {
@@ -71,7 +71,7 @@ UA_Policy_New_Context(UA_SecurityPolicy * securityPolicy,
     if(retval != UA_STATUSCODE_GOOD) {
         EVP_PKEY_free(context->localPrivateKey);
         UA_free(context);
-        return retval; 
+        return retval;
     }
 
     context->logger = logger;
@@ -92,18 +92,67 @@ UA_Policy_Clear_Context(UA_SecurityPolicy *policy) {
         (Policy_Context_Basic256Sha256 *) policy->policyContext;
     EVP_PKEY_free(pc->localPrivateKey);
     UA_ByteString_clear(&pc->localCertThumbprint);
-    UA_free(pc);        
+    UA_free(pc);
     return;
 }
 
+static UA_StatusCode
+updateCertificateAndPrivateKey_sp_basic256sha256(UA_SecurityPolicy *securityPolicy,
+                                           const UA_ByteString newCertificate,
+                                           const UA_ByteString newPrivateKey) {
+    if(securityPolicy == NULL)
+        return UA_STATUSCODE_BADINTERNALERROR;
+
+    if(securityPolicy->policyContext == NULL)
+        return UA_STATUSCODE_BADINTERNALERROR;
+
+    Policy_Context_Basic256Sha256 *pc =
+        (Policy_Context_Basic256Sha256 *)securityPolicy->policyContext;
+
+    UA_ByteString_clear(&securityPolicy->localCertificate);
+
+    UA_StatusCode retval = UA_OpenSSL_LoadLocalCertificate(
+        &newCertificate, &securityPolicy->localCertificate);
+
+    if(retval != UA_STATUSCODE_GOOD)
+        return retval;
+
+    /* Set the new private key */
+    EVP_PKEY_free(pc->localPrivateKey);
+
+    pc->localPrivateKey = UA_OpenSSL_LoadPrivateKey(&newPrivateKey);
+
+    if(!pc->localPrivateKey) {
+        retval = UA_STATUSCODE_BADSECURITYCHECKSFAILED;
+        goto error;
+    }
+
+    UA_ByteString_clear(&pc->localCertThumbprint);
+
+    retval = UA_Openssl_X509_GetCertificateThumbprint(&securityPolicy->localCertificate,
+                                                      &pc->localCertThumbprint, true);
+    if(retval != UA_STATUSCODE_GOOD) {
+        goto error;
+    }
+
+    return retval;
+
+error:
+    UA_LOG_ERROR(securityPolicy->logger, UA_LOGCATEGORY_SECURITYPOLICY,
+                 "Could not update certificate and private key");
+    if(securityPolicy->policyContext != NULL)
+        UA_Policy_Clear_Context(securityPolicy);
+    return retval;
+}
+
 /* create the channel context */
 
-static UA_StatusCode 
+static UA_StatusCode
 UA_ChannelModule_New_Context(const UA_SecurityPolicy * securityPolicy,
                              const UA_ByteString * remoteCertificate,
                              void ** channelContext) {
     if(securityPolicy == NULL || remoteCertificate == NULL || channelContext == NULL)
-        return UA_STATUSCODE_BADINTERNALERROR;                                  
+        return UA_STATUSCODE_BADINTERNALERROR;
 
     Channel_Context_Basic256Sha256 *context = (Channel_Context_Basic256Sha256 *)
         UA_malloc(sizeof(Channel_Context_Basic256Sha256));
@@ -127,7 +176,7 @@ UA_ChannelModule_New_Context(const UA_SecurityPolicy * securityPolicy,
     /* decode to X509 */
     context->remoteCertificateX509 = UA_OpenSSL_LoadCertificate(&context->remoteCertificate);
     if(context->remoteCertificateX509 == NULL) {
-        UA_ByteString_clear(&context->remoteCertificate); 
+        UA_ByteString_clear(&context->remoteCertificate);
         UA_free(context);
         return UA_STATUSCODE_BADINTERNALERROR;
     }
@@ -136,7 +185,7 @@ UA_ChannelModule_New_Context(const UA_SecurityPolicy * securityPolicy,
         (Policy_Context_Basic256Sha256 *)securityPolicy->policyContext;
     *channelContext = context;
 
-    UA_LOG_INFO(securityPolicy->logger, UA_LOGCATEGORY_SECURITYPOLICY, 
+    UA_LOG_INFO(securityPolicy->logger, UA_LOGCATEGORY_SECURITYPOLICY,
                 "The basic256sha256 security policy channel with openssl is created.");
 
     return UA_STATUSCODE_GOOD;
@@ -144,30 +193,30 @@ UA_ChannelModule_New_Context(const UA_SecurityPolicy * securityPolicy,
 
 /* delete the channel context */
 
-static void 
+static void
 UA_ChannelModule_Delete_Context(void * channelContext) {
     if(!channelContext)
         return;
-    
+
     Channel_Context_Basic256Sha256 * cc = (Channel_Context_Basic256Sha256 *)channelContext;
-    X509_free(cc->remoteCertificateX509);                                           
-    UA_ByteString_clear(&cc->remoteCertificate); 
+    X509_free(cc->remoteCertificateX509);
+    UA_ByteString_clear(&cc->remoteCertificate);
     UA_ByteString_clear(&cc->localSymSigningKey);
     UA_ByteString_clear(&cc->localSymEncryptingKey);
     UA_ByteString_clear(&cc->localSymIv);
     UA_ByteString_clear(&cc->remoteSymSigningKey);
     UA_ByteString_clear(&cc->remoteSymEncryptingKey);
     UA_ByteString_clear(&cc->remoteSymIv);
-    
-    UA_LOG_INFO(cc->policyContext->logger, UA_LOGCATEGORY_SECURITYPOLICY, 
-                "The basic256sha256 security policy channel with openssl is deleted.");   
+
+    UA_LOG_INFO(cc->policyContext->logger, UA_LOGCATEGORY_SECURITYPOLICY,
+                "The basic256sha256 security policy channel with openssl is deleted.");
     UA_free(cc);
 }
 
 /* Verifies the signature of the message using the provided keys in the context.
  * AsymmetricSignatureAlgorithm_RSA-PKCS15-SHA2-256 */
-static UA_StatusCode 
-UA_AsySig_Basic256Sha256_Verify(void *channelContext, 
+static UA_StatusCode
+UA_AsySig_Basic256Sha256_Verify(void *channelContext,
                                 const UA_ByteString *message,
                                 const UA_ByteString *signature) {
     if(message == NULL || signature == NULL || channelContext == NULL)
@@ -179,8 +228,8 @@ UA_AsySig_Basic256Sha256_Verify(void *channelContext,
                                                   signature);
 }
 
-/* Compares the supplied certificate with the certificate 
- * in the endpoint context 
+/* Compares the supplied certificate with the certificate
+ * in the endpoint context
  */
 
 static UA_StatusCode
@@ -197,14 +246,14 @@ UA_compareCertificateThumbprint(const UA_SecurityPolicy * securityPolicy,
 
 /* Generates a thumbprint for the specified certificate */
 
-static UA_StatusCode 
+static UA_StatusCode
 UA_makeCertificateThumbprint(const UA_SecurityPolicy * securityPolicy,
                              const UA_ByteString * certificate,
                              UA_ByteString * thumbprint) {
-    return UA_Openssl_X509_GetCertificateThumbprint(certificate, thumbprint, false); 
+    return UA_Openssl_X509_GetCertificateThumbprint(certificate, thumbprint, false);
 }
 
-static UA_StatusCode 
+static UA_StatusCode
 UA_Asym_Basic256Sha256_Decrypt(void * channelContext, UA_ByteString * data) {
     if(channelContext == NULL || data == NULL)
         return UA_STATUSCODE_BADINVALIDARGUMENT;
@@ -223,10 +272,10 @@ UA_Asym_Basic256Sha256_getRemoteSignatureSize(const void *channelContext) {
         (const Channel_Context_Basic256Sha256 *) channelContext;
     UA_Int32 keyLen = 0;
     UA_Openssl_RSA_Public_GetKeyLength(cc->remoteCertificateX509, &keyLen);
-    return (size_t) keyLen; 
+    return (size_t) keyLen;
 }
 
-static size_t 
+static size_t
 UA_AsySig_Basic256Sha256_getLocalSignatureSize(const void *channelContext) {
     if(channelContext == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
@@ -236,7 +285,7 @@ UA_AsySig_Basic256Sha256_getLocalSignatureSize(const void *channelContext) {
     Policy_Context_Basic256Sha256 * pc = cc->policyContext;
     UA_Int32 keyLen = 0;
     UA_Openssl_RSA_Private_GetKeyLength(pc->localPrivateKey, &keyLen);
-    return (size_t) keyLen; 
+    return (size_t) keyLen;
 }
 
 static size_t
@@ -251,7 +300,7 @@ UA_AsymEn_Basic256Sha256_getRemotePlainTextBlockSize(const void *channelContext)
     return (size_t) keyLen - UA_SECURITYPOLICY_BASIC256SHA256_RSAPADDING_LEN;
 }
 
-static size_t 
+static size_t
 UA_AsymEn_Basic256Sha256_getRemoteBlockSize(const void *channelContext) {
     if(channelContext == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
@@ -281,19 +330,19 @@ UA_Sym_Basic256Sha256_generateNonce(void *policyContext,
     UA_Int32 rc = RAND_bytes(out->data, (int) out->length);
     if(rc != 1)
         return UA_STATUSCODE_BADUNEXPECTEDERROR;
-    return UA_STATUSCODE_GOOD;                                         
+    return UA_STATUSCODE_GOOD;
 }
 
-static size_t 
+static size_t
 UA_SymEn_Basic256Sha256_getLocalKeyLength(const void *channelContext) {
     /* 32 bytes 256 bits */
-    return UA_SECURITYPOLICY_BASIC256SHA256_SYM_ENCRYPTION_KEY_LENGTH; 
+    return UA_SECURITYPOLICY_BASIC256SHA256_SYM_ENCRYPTION_KEY_LENGTH;
 }
 
-static size_t 
+static size_t
 UA_SymSig_Basic256Sha256_getLocalKeyLength(const void *channelContext) {
-    /* 32 bytes 256 bits */                                                
-    return UA_SECURITYPOLICY_BASIC256SHA256_SYM_SIGNING_KEY_LENGTH; 
+    /* 32 bytes 256 bits */
+    return UA_SECURITYPOLICY_BASIC256SHA256_SYM_SIGNING_KEY_LENGTH;
 }
 
 static UA_StatusCode
@@ -337,7 +386,7 @@ UA_ChannelM_Basic256Sha256_setLocalSymIv(void * channelContext,
 static size_t
 UA_SymEn_Basic256Sha256_getRemoteKeyLength(const void * channelContext) {
     /* 32 bytes 256 bits */
-    return UA_SECURITYPOLICY_BASIC256SHA256_SYM_ENCRYPTION_KEY_LENGTH; 
+    return UA_SECURITYPOLICY_BASIC256SHA256_SYM_ENCRYPTION_KEY_LENGTH;
 }
 
 static size_t
@@ -347,7 +396,7 @@ UA_SymEn_Basic256Sha256_getBlockSize(const void *channelContext) {
 
 static size_t
 UA_SymSig_Basic256Sha256_getRemoteKeyLength(const void * channelContext) {
-    /* 32 bytes 256 bits */                                                
+    /* 32 bytes 256 bits */
     return UA_SECURITYPOLICY_BASIC256SHA256_SYM_SIGNING_KEY_LENGTH;
 }
 
@@ -385,7 +434,7 @@ static UA_StatusCode
 UA_AsySig_Basic256Sha256_sign(void *channelContext, const UA_ByteString * message,
                               UA_ByteString *signature) {
     if(channelContext == NULL || message == NULL || signature == NULL)
-        return UA_STATUSCODE_BADINTERNALERROR; 
+        return UA_STATUSCODE_BADINTERNALERROR;
     Channel_Context_Basic256Sha256 *cc = (Channel_Context_Basic256Sha256 *) channelContext;
     Policy_Context_Basic256Sha256 *pc = cc->policyContext;
     return UA_Openssl_RSA_PKCS1_V15_SHA256_Sign(message, pc->localPrivateKey, signature);
@@ -394,9 +443,9 @@ UA_AsySig_Basic256Sha256_sign(void *channelContext, const UA_ByteString * messag
 static UA_StatusCode
 UA_AsymEn_Basic256Sha256_encrypt(void *channelContext, UA_ByteString *data) {
     if(channelContext == NULL || data == NULL)
-        return UA_STATUSCODE_BADINTERNALERROR; 
+        return UA_STATUSCODE_BADINTERNALERROR;
     Channel_Context_Basic256Sha256 * cc =
-        (Channel_Context_Basic256Sha256 *) channelContext;    
+        (Channel_Context_Basic256Sha256 *) channelContext;
     return UA_Openssl_RSA_OAEP_Encrypt(data, UA_SECURITYPOLICY_BASIC256SHA256_RSAPADDING_LEN,
                                        cc->remoteCertificateX509);
 }
@@ -411,17 +460,17 @@ UA_SymSig_Basic256Sha256_verify(void *channelContext, const UA_ByteString *messa
                                 const UA_ByteString *signature) {
     if(channelContext == NULL || message == NULL || signature == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
-    
+
     Channel_Context_Basic256Sha256 * cc = (Channel_Context_Basic256Sha256 *) channelContext;
-    return UA_OpenSSL_HMAC_SHA256_Verify(message, &cc->remoteSymSigningKey, signature);   
+    return UA_OpenSSL_HMAC_SHA256_Verify(message, &cc->remoteSymSigningKey, signature);
 }
 
-static UA_StatusCode 
+static UA_StatusCode
 UA_SymSig_Basic256Sha256_sign(void *channelContext, const UA_ByteString *message,
                               UA_ByteString *signature) {
     if(channelContext == NULL || message == NULL || signature == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
-    
+
     Channel_Context_Basic256Sha256 * cc = (Channel_Context_Basic256Sha256 *) channelContext;
     return UA_OpenSSL_HMAC_SHA256_Sign(message, &cc->localSymSigningKey, signature);
 }
@@ -444,7 +493,7 @@ static UA_StatusCode
 UA_SymEn_Basic256Sha256_encrypt(void *channelContext, UA_ByteString *data) {
     if(channelContext == NULL || data == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
-    
+
     Channel_Context_Basic256Sha256 * cc = (Channel_Context_Basic256Sha256 *) channelContext;
     return UA_OpenSSL_AES_256_CBC_Encrypt(&cc->localSymIv, &cc->localSymEncryptingKey, data);
 }
@@ -454,13 +503,13 @@ UA_ChannelM_Basic256Sha256_compareCertificate(const void *channelContext,
                                               const UA_ByteString *certificate) {
     if(channelContext == NULL || certificate == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
-    
+
     const Channel_Context_Basic256Sha256 * cc =
         (const Channel_Context_Basic256Sha256 *) channelContext;
     return UA_OpenSSL_X509_compare(certificate, cc->remoteCertificateX509);
 }
 
-static size_t 
+static size_t
 UA_AsymEn_Basic256Sha256_getLocalKeyLength(const void *channelContext) {
     if(channelContext == NULL)
         return UA_STATUSCODE_BADINTERNALERROR;
@@ -470,7 +519,7 @@ UA_AsymEn_Basic256Sha256_getLocalKeyLength(const void *channelContext) {
     Policy_Context_Basic256Sha256 *pc = cc->policyContext;
     UA_Int32 keyLen = 0;
     UA_Openssl_RSA_Private_GetKeyLength(pc->localPrivateKey, &keyLen);
-    return (size_t) keyLen * 8; 
+    return (size_t) keyLen * 8;
 }
 
 /* the main entry of Basic256Sha256 */
@@ -478,12 +527,12 @@ UA_AsymEn_Basic256Sha256_getLocalKeyLength(const void *channelContext) {
 UA_StatusCode
 UA_SecurityPolicy_Basic256Sha256(UA_SecurityPolicy *policy,
                                  const UA_ByteString localCertificate,
-                                 const UA_ByteString localPrivateKey, 
+                                 const UA_ByteString localPrivateKey,
                                  const UA_Logger *logger) {
     UA_SecurityPolicyAsymmetricModule *asymmetricModule = &policy->asymmetricModule;
     UA_SecurityPolicySymmetricModule *symmetricModule = &policy->symmetricModule;
-    UA_SecurityPolicyChannelModule *channelModule = &policy->channelModule; 
-    UA_LOG_INFO(logger, UA_LOGCATEGORY_SECURITYPOLICY, 
+    UA_SecurityPolicyChannelModule *channelModule = &policy->channelModule;
+    UA_LOG_INFO(logger, UA_LOGCATEGORY_SECURITYPOLICY,
                 "The basic256sha256 security policy with openssl is added.");
 
     UA_Openssl_Init();
@@ -531,7 +580,7 @@ UA_SecurityPolicy_Basic256Sha256(UA_SecurityPolicy *policy,
     asymEncryAlg->getLocalKeyLength = UA_AsymEn_Basic256Sha256_getLocalKeyLength;
     asymEncryAlg->getRemoteKeyLength = UA_AsymEn_Basic256Sha256_getRemoteKeyLength;
     asymEncryAlg->getRemoteBlockSize = UA_AsymEn_Basic256Sha256_getRemoteBlockSize;
-    asymEncryAlg->getRemotePlainTextBlockSize = 
+    asymEncryAlg->getRemotePlainTextBlockSize =
             UA_AsymEn_Basic256Sha256_getRemotePlainTextBlockSize;
 
     /* AsymmetricModule */
@@ -547,7 +596,7 @@ UA_SecurityPolicy_Basic256Sha256(UA_SecurityPolicy *policy,
     UA_SecurityPolicyEncryptionAlgorithm *symEncryptionAlgorithm =
         &symmetricModule->cryptoModule.encryptionAlgorithm;
     symEncryptionAlgorithm->uri =
-        UA_STRING("http://www.w3.org/2001/04/xmlenc#aes256-cbc\0");        
+        UA_STRING("http://www.w3.org/2001/04/xmlenc#aes256-cbc\0");
     symEncryptionAlgorithm->encrypt = UA_SymEn_Basic256Sha256_encrypt;
     symEncryptionAlgorithm->decrypt = UA_SymEn_Basic256Sha256_decrypt;
     symEncryptionAlgorithm->getLocalKeyLength = UA_SymEn_Basic256Sha256_getLocalKeyLength;
@@ -564,13 +613,15 @@ UA_SecurityPolicy_Basic256Sha256(UA_SecurityPolicy *policy,
     symSignatureAlgorithm->sign = UA_SymSig_Basic256Sha256_sign;
     symSignatureAlgorithm->getLocalSignatureSize =
         UA_SymSig_Basic256Sha256_getLocalSignatureSize;
-    symSignatureAlgorithm->getRemoteSignatureSize = 
+    symSignatureAlgorithm->getRemoteSignatureSize =
         UA_SymSig_Basic256Sha256_getRemoteSignatureSize;
     symSignatureAlgorithm->getLocalKeyLength =
         UA_SymSig_Basic256Sha256_getLocalKeyLength;
     symSignatureAlgorithm->getRemoteKeyLength =
         UA_SymSig_Basic256Sha256_getRemoteKeyLength;
 
+    policy->updateCertificateAndPrivateKey =
+        updateCertificateAndPrivateKey_sp_basic256sha256;
     policy->clear = UA_Policy_Clear_Context;
     retval = UA_Policy_New_Context(policy, localPrivateKey, logger);
     if(retval != UA_STATUSCODE_GOOD) {
