@@ -76,13 +76,13 @@ UA_StatusCode
 UA_Client_getEndpoints(UA_Client *client, const char *serverUrl,
                        size_t *endpointDescriptionsSize,
                        UA_EndpointDescription** endpointDescriptions) {
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
 
     UA_Boolean connected = (client->channel.state == UA_SECURECHANNELSTATE_OPEN);
     /* Client is already connected to a different server */
     if(connected && strncmp((const char*)client->config.endpoint.endpointUrl.data, serverUrl,
                             client->config.endpoint.endpointUrl.length) != 0) {
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         return UA_STATUSCODE_BADINVALIDARGUMENT;
     }
 
@@ -91,13 +91,13 @@ UA_Client_getEndpoints(UA_Client *client, const char *serverUrl,
     if(!connected) {
         retval = connectSecureChannel(client, serverUrl);
         if(retval != UA_STATUSCODE_GOOD) {
-            UA_UNLOCK(&client->clientMutex);
+            unlockClient(client);
             return retval;
         }
     }
     retval = getEndpointsInternal(client, url, endpointDescriptionsSize,
                                   endpointDescriptions);
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
 
     if(!connected)
         UA_Client_disconnect(client);
@@ -110,12 +110,12 @@ UA_Client_findServers(UA_Client *client, const char *serverUrl,
                       size_t localeIdsSize, UA_String *localeIds,
                       size_t *registeredServersSize,
                       UA_ApplicationDescription **registeredServers) {
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     UA_Boolean connected = (client->channel.state == UA_SECURECHANNELSTATE_OPEN);
     /* Client is already connected to a different server */
     if(connected && strncmp((const char*)client->config.endpoint.endpointUrl.data, serverUrl,
                             client->config.endpoint.endpointUrl.length) != 0) {
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         return UA_STATUSCODE_BADINVALIDARGUMENT;
     }
 
@@ -123,7 +123,7 @@ UA_Client_findServers(UA_Client *client, const char *serverUrl,
     if(!connected) {
         retval = connectSecureChannel(client, serverUrl);
         if(retval != UA_STATUSCODE_GOOD) {
-            UA_UNLOCK(&client->clientMutex);
+            unlockClient(client);
             return retval;
         }
     }
@@ -141,7 +141,7 @@ UA_Client_findServers(UA_Client *client, const char *serverUrl,
     __Client_Service(client, &request, &UA_TYPES[UA_TYPES_FINDSERVERSREQUEST],
                      &response, &UA_TYPES[UA_TYPES_FINDSERVERSRESPONSE]);
 
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
 
     /* Process the response */
     retval = response.responseHeader.serviceResult;
@@ -167,13 +167,13 @@ UA_Client_findServersOnNetwork(UA_Client *client, const char *serverUrl,
                                UA_UInt32 startingRecordId, UA_UInt32 maxRecordsToReturn,
                                size_t serverCapabilityFilterSize, UA_String *serverCapabilityFilter,
                                size_t *serverOnNetworkSize, UA_ServerOnNetwork **serverOnNetwork) {
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
 
     UA_Boolean connected = (client->channel.state == UA_SECURECHANNELSTATE_OPEN);
     /* Client is already connected to a different server */
     if(connected && strncmp((const char*)client->config.endpoint.endpointUrl.data, serverUrl,
                             client->config.endpoint.endpointUrl.length) != 0) {
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         return UA_STATUSCODE_BADINVALIDARGUMENT;
     }
 
@@ -181,7 +181,7 @@ UA_Client_findServersOnNetwork(UA_Client *client, const char *serverUrl,
     if(!connected) {
         retval = connectSecureChannel(client, serverUrl);
         if(retval != UA_STATUSCODE_GOOD) {
-            UA_LOCK(&client->clientMutex);
+            lockClient(client);
             return retval;
         }
     }
@@ -199,7 +199,7 @@ UA_Client_findServersOnNetwork(UA_Client *client, const char *serverUrl,
     __Client_Service(client, &request, &UA_TYPES[UA_TYPES_FINDSERVERSONNETWORKREQUEST],
                      &response, &UA_TYPES[UA_TYPES_FINDSERVERSONNETWORKRESPONSE]);
 
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
 
     /* Process the response */
     retval = response.responseHeader.serviceResult;
