@@ -71,6 +71,8 @@ ua_Subscriptions_create(UA_Client *client, UA_Client_Subscription *newSub,
 static void
 ua_Subscriptions_create_handler(UA_Client *client, void *data,
                                 UA_UInt32 requestId, void *r) {
+    UA_LOCK_ASSERT(&client->clientMutex, 1);
+
     UA_CreateSubscriptionResponse *response = (UA_CreateSubscriptionResponse *)r;
     CustomCallback *cc = (CustomCallback *)data;
     UA_Client_Subscription *newSub = (UA_Client_Subscription *)cc->clientData;
@@ -80,9 +82,7 @@ ua_Subscriptions_create_handler(UA_Client *client, void *data,
     }
 
     /* Prepare the internal representation */
-    UA_LOCK(&client->clientMutex);
     ua_Subscriptions_create(client, newSub, response);
-    UA_UNLOCK(&client->clientMutex);
 
 cleanup:
     if(cc->userCallback)
@@ -96,12 +96,15 @@ UA_Client_Subscriptions_create(UA_Client *client,
                                void *subscriptionContext,
                                UA_Client_StatusChangeNotificationCallback statusChangeCallback,
                                UA_Client_DeleteSubscriptionCallback deleteCallback) {
+    lockClient(client);
+
     UA_CreateSubscriptionResponse response;
     UA_Client_Subscription *sub = (UA_Client_Subscription *)
         UA_malloc(sizeof(UA_Client_Subscription));
     if(!sub) {
         UA_CreateSubscriptionResponse_init(&response);
         response.responseHeader.serviceResult = UA_STATUSCODE_BADOUTOFMEMORY;
+        unlockClient(client);
         return response;
     }
     sub->context = subscriptionContext;
@@ -109,18 +112,17 @@ UA_Client_Subscriptions_create(UA_Client *client,
     sub->deleteCallback = deleteCallback;
 
     /* Send the request as a synchronous service call */
-    __UA_Client_Service(client,
-                        &request, &UA_TYPES[UA_TYPES_CREATESUBSCRIPTIONREQUEST],
-                        &response, &UA_TYPES[UA_TYPES_CREATESUBSCRIPTIONRESPONSE]);
-    if (response.responseHeader.serviceResult != UA_STATUSCODE_GOOD) {
-        UA_free (sub);
+    __Client_Service(client, &request, &UA_TYPES[UA_TYPES_CREATESUBSCRIPTIONREQUEST],
+                     &response, &UA_TYPES[UA_TYPES_CREATESUBSCRIPTIONRESPONSE]);
+    if(response.responseHeader.serviceResult != UA_STATUSCODE_GOOD) {
+        UA_free(sub);
+        unlockClient(client);
         return response;
     }
 
-    UA_LOCK(&client->clientMutex);
     ua_Subscriptions_create(client, sub, &response);
-    UA_UNLOCK(&client->clientMutex);
 
+    unlockClient(client);
     return response;
 }
 
@@ -177,11 +179,11 @@ ua_Subscriptions_modify(UA_Client *client, UA_Client_Subscription *sub,
 static void
 ua_Subscriptions_modify_handler(UA_Client *client, void *data, UA_UInt32 requestId,
                                 void *r) {
+    UA_LOCK_ASSERT(&client->clientMutex, 1);
+
     UA_ModifySubscriptionResponse *response = (UA_ModifySubscriptionResponse *)r;
     CustomCallback *cc = (CustomCallback *)data;
-    UA_LOCK(&client->clientMutex);
-    UA_Client_Subscription *sub =
-        findSubscription(client, (UA_UInt32)(uintptr_t)cc->clientData);
+    UA_Client_Subscription *sub = findSubscription(client, (UA_UInt32)(uintptr_t)cc->clientData);
     if(sub) {
         ua_Subscriptions_modify(client, sub, response);
     } else {
@@ -189,7 +191,6 @@ ua_Subscriptions_modify_handler(UA_Client *client, void *data, UA_UInt32 request
                     "No internal representation of subscription %" PRIu32,
                     (UA_UInt32)(uintptr_t)cc->clientData);
     }
-    UA_UNLOCK(&client->clientMutex);
 
     if(cc->userCallback)
         cc->userCallback(client, cc->userData, requestId, response);
@@ -203,29 +204,28 @@ UA_Client_Subscriptions_modify(UA_Client *client,
     UA_ModifySubscriptionResponse_init(&response);
 
     /* Find the internal representation */
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     UA_Client_Subscription *sub = findSubscription(client, request.subscriptionId);
-    UA_UNLOCK(&client->clientMutex);
     if(!sub) {
+        unlockClient(client);
         response.responseHeader.serviceResult = UA_STATUSCODE_BADSUBSCRIPTIONIDINVALID;
         return response;
     }
 
     /* Call the service */
-    __UA_Client_Service(client,
-                        &request, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONREQUEST],
-                        &response, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONRESPONSE]);
+    __Client_Service(client,
+                     &request, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONREQUEST],
+                     &response, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONRESPONSE]);
 
     /* Adjust the internal representation. Lookup again for thread-safety. */
-    UA_LOCK(&client->clientMutex);
     sub = findSubscription(client, request.subscriptionId);
     if(!sub) {
         response.responseHeader.serviceResult = UA_STATUSCODE_BADSUBSCRIPTIONIDINVALID;
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         return response;
     }
     ua_Subscriptions_modify(client, sub, &response);
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     return response;
 }
 
@@ -234,24 +234,32 @@ UA_Client_Subscriptions_modify_async(UA_Client *client,
                                      const UA_ModifySubscriptionRequest request,
                                      UA_ClientAsyncServiceCallback callback,
                                      void *userdata, UA_UInt32 *requestId) {
+    lockClient(client);
+
     /* Find the internal representation */
-    UA_LOCK(&client->clientMutex);
     UA_Client_Subscription *sub = findSubscription(client, request.subscriptionId);
-    UA_UNLOCK(&client->clientMutex);
-    if(!sub)
+    if(!sub) {
+        unlockClient(client);
         return UA_STATUSCODE_BADSUBSCRIPTIONIDINVALID;
+    }
 
     CustomCallback *cc = (CustomCallback *)UA_calloc(1, sizeof(CustomCallback));
-    if(!cc)
+    if(!cc) {
+        unlockClient(client);
         return UA_STATUSCODE_BADOUTOFMEMORY;
+    }
 
     cc->clientData = (void *)(uintptr_t)request.subscriptionId;
     cc->userData = userdata;
     cc->userCallback = callback;
 
-    return __UA_Client_AsyncService(client, &request, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONREQUEST],
-                                    ua_Subscriptions_modify_handler, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONRESPONSE],
-                                    cc, requestId);
+    UA_StatusCode res =
+        __Client_AsyncService(client, &request, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONREQUEST],
+                              ua_Subscriptions_modify_handler, &UA_TYPES[UA_TYPES_MODIFYSUBSCRIPTIONRESPONSE],
+                              cc, requestId);
+
+    unlockClient(client);
+    return res;
 }
 
 static void *
@@ -283,9 +291,7 @@ __Client_Subscription_deleteInternal(UA_Client *client,
     if(sub->deleteCallback) {
         void *subC = sub->context;
         UA_UInt32 subId = sub->subscriptionId;
-        UA_UNLOCK(&client->clientMutex);
         sub->deleteCallback(client, subId, subC);
-        UA_LOCK(&client->clientMutex);
     }
 
     /* Remove */
@@ -338,10 +344,10 @@ ua_Subscriptions_delete_handler(UA_Client *client, void *data,
     DeleteSubscriptionCallback *dsc =
         (DeleteSubscriptionCallback*)data;
 
+    lockClient(client);
+
     /* Delete */
-    UA_LOCK(&client->clientMutex);
     __Client_Subscription_processDelete(client, &dsc->request, response);
-    UA_UNLOCK(&client->clientMutex);
 
     /* Userland Callback */
     dsc->userCallback(client, dsc->userData, requestId, response);
@@ -349,6 +355,8 @@ ua_Subscriptions_delete_handler(UA_Client *client, void *data,
     /* Cleanup */
     UA_DeleteSubscriptionsRequest_clear(&dsc->request);
     UA_free(dsc);
+
+    unlockClient(client);
 }
 
 UA_StatusCode
@@ -378,15 +386,17 @@ UA_Client_Subscriptions_delete_async(UA_Client *client,
 UA_DeleteSubscriptionsResponse
 UA_Client_Subscriptions_delete(UA_Client *client,
                                const UA_DeleteSubscriptionsRequest request) {
+    lockClient(client);
+
     /* Send the request */
     UA_DeleteSubscriptionsResponse response;
-    __UA_Client_Service(client, &request, &UA_TYPES[UA_TYPES_DELETESUBSCRIPTIONSREQUEST],
-                        &response, &UA_TYPES[UA_TYPES_DELETESUBSCRIPTIONSRESPONSE]);
+    __Client_Service(client, &request, &UA_TYPES[UA_TYPES_DELETESUBSCRIPTIONSREQUEST],
+                     &response, &UA_TYPES[UA_TYPES_DELETESUBSCRIPTIONSRESPONSE]);
 
     /* Process */
-    UA_LOCK(&client->clientMutex);
     __Client_Subscription_processDelete(client, &request, &response);
-    UA_UNLOCK(&client->clientMutex);
+
+    unlockClient(client);
     return response;
 }
 
@@ -431,9 +441,7 @@ MonitoredItem_delete(UA_Client *client, UA_Client_Subscription *sub,
         void *monC = mon->context;
         UA_UInt32 subId = sub->subscriptionId;
         UA_UInt32 monId = mon->monitoredItemId;
-        UA_UNLOCK(&client->clientMutex);
         mon->deleteCallback(client, subId, subC, monId, monC);
-        UA_LOCK(&client->clientMutex);
     }
     UA_free(mon);
 }
@@ -480,10 +488,8 @@ ua_MonitoredItems_create(UA_Client *client, MonitoredItems_CreateData *data,
         if(response->results[i].statusCode != UA_STATUSCODE_GOOD) {
             void *subC = sub->context;
             UA_UInt32 subId = sub->subscriptionId;
-            UA_UNLOCK(&client->clientMutex);
             if(deleteCallbacks[i])
                 deleteCallbacks[i](client, subId, subC, 0, data->contexts[i]);
-            UA_LOCK(&client->clientMutex);
             continue;
         }
 
@@ -492,10 +498,8 @@ ua_MonitoredItems_create(UA_Client *client, MonitoredItems_CreateData *data,
         if(!newMon) {
             void *subC = sub->context;
             UA_UInt32 subId = sub->subscriptionId;
-            UA_UNLOCK(&client->clientMutex);
             if(deleteCallbacks[i])
                 deleteCallbacks[i](client, subId, subC, 0, data->contexts[i]);
-            UA_LOCK(&client->clientMutex);
             continue;
         }
 
@@ -520,11 +524,9 @@ ua_MonitoredItems_create(UA_Client *client, MonitoredItems_CreateData *data,
  cleanup:
     for(size_t i = 0; i < request->itemsToCreateSize; i++) {
         void *subC = sub ? sub->context : NULL;
-        UA_UNLOCK(&client->clientMutex);
         if(deleteCallbacks[i])
             deleteCallbacks[i](client, data->request.subscriptionId,
                                subC, 0, data->contexts[i]);
-        UA_LOCK(&client->clientMutex);
     }
 }
 
@@ -534,15 +536,17 @@ ua_MonitoredItems_create_async_handler(UA_Client *client, void *d, UA_UInt32 req
     UA_CreateMonitoredItemsResponse *response = (UA_CreateMonitoredItemsResponse *)r;
     MonitoredItems_CreateData *data = (MonitoredItems_CreateData *)d;
 
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
+
     ua_MonitoredItems_create(client, data, response);
     MonitoredItems_CreateData_clear(client, data);
-    UA_UNLOCK(&client->clientMutex);
 
     if(data->userCallback)
         data->userCallback(client, data->userData, requestId, response);
 
     UA_free(data);
+
+    unlockClient(client);
 }
 
 static UA_StatusCode
@@ -676,10 +680,10 @@ UA_Client_MonitoredItems_createDataChanges(UA_Client *client,
                                            UA_Client_DataChangeNotificationCallback *callbacks,
                                            UA_Client_DeleteMonitoredItemCallback *deleteCallbacks) {
     UA_CreateMonitoredItemsResponse response;
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     ua_Client_MonitoredItems_create(client, &request, contexts, (void **)callbacks,
                                     deleteCallbacks, &response);
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     return response;
 }
 
@@ -691,11 +695,11 @@ UA_Client_MonitoredItems_createDataChanges_async(UA_Client *client,
                                                  UA_Client_DeleteMonitoredItemCallback *deleteCallbacks,
                                                  UA_ClientAsyncServiceCallback createCallback,
                                                  void *userdata, UA_UInt32 *requestId) {
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     UA_StatusCode res =
         createDataChanges_async(client, request, contexts, (void **)callbacks,
                                 deleteCallbacks, createCallback, userdata, requestId);
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     return res;
 }
 
@@ -737,10 +741,10 @@ UA_Client_MonitoredItems_createEvents(UA_Client *client,
                                       UA_Client_EventNotificationCallback *callback,
                                       UA_Client_DeleteMonitoredItemCallback *deleteCallback) {
     UA_CreateMonitoredItemsResponse response;
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     ua_Client_MonitoredItems_create(client, &request, contexts, (void **)callback,
                                     deleteCallback, &response);
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     return response;
 }
 
@@ -753,11 +757,11 @@ UA_Client_MonitoredItems_createEvents_async(UA_Client *client,
                                             UA_Client_DeleteMonitoredItemCallback *deleteCallbacks,
                                             UA_ClientAsyncServiceCallback createCallback,
                                             void *userdata, UA_UInt32 *requestId) {
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     UA_StatusCode res =
         createDataChanges_async(client, request, contexts, (void **)callbacks, deleteCallbacks,
                                 createCallback, userdata, requestId);
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     return res;
 }
 
@@ -823,7 +827,7 @@ ua_MonitoredItems_delete_handler(UA_Client *client, void *d, UA_UInt32 requestId
     UA_DeleteMonitoredItemsRequest *request =
         (UA_DeleteMonitoredItemsRequest *)cc->clientData;
 
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
 
     if(response->responseHeader.serviceResult != UA_STATUSCODE_GOOD)
         goto cleanup;
@@ -840,11 +844,12 @@ ua_MonitoredItems_delete_handler(UA_Client *client, void *d, UA_UInt32 requestId
     ua_MonitoredItems_delete(client, sub, request, response);
 
 cleanup:
-    UA_UNLOCK(&client->clientMutex);
     if(cc->userCallback)
         cc->userCallback(client, cc->userData, requestId, response);
     UA_DeleteMonitoredItemsRequest_delete(request);
     UA_free(cc);
+
+    unlockClient(client);
 }
 
 UA_DeleteMonitoredItemsResponse
@@ -859,7 +864,7 @@ UA_Client_MonitoredItems_delete(UA_Client *client,
     if(response.responseHeader.serviceResult != UA_STATUSCODE_GOOD)
         return response;
 
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
 
     /* Find the internal subscription representation */
     UA_Client_Subscription *sub = findSubscription(client, request.subscriptionId);
@@ -867,14 +872,14 @@ UA_Client_MonitoredItems_delete(UA_Client *client,
         UA_LOG_INFO(client->config.logging, UA_LOGCATEGORY_CLIENT,
                     "No internal representation of subscription %" PRIu32,
                     request.subscriptionId);
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         return response;
     }
 
     /* Remove MonitoredItems in the internal representation */
     ua_MonitoredItems_delete(client, sub, &request, &response);
 
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
 
     return response;
 }
@@ -959,10 +964,10 @@ UA_Client_MonitoredItems_modify(UA_Client *client,
     UA_ModifyMonitoredItemsResponse response;
     UA_ModifyMonitoredItemsResponse_init(&response);
 
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     UA_Client_Subscription *sub = findSubscription(client, request.subscriptionId);
     if(!sub) {
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         response.responseHeader.serviceResult = UA_STATUSCODE_BADSUBSCRIPTIONIDINVALID;
         return response;
     }
@@ -975,7 +980,7 @@ UA_Client_MonitoredItems_modify(UA_Client *client,
                      &UA_TYPES[UA_TYPES_MODIFYMONITOREDITEMSREQUEST], &response,
                      &UA_TYPES[UA_TYPES_MODIFYMONITOREDITEMSRESPONSE]);
 
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     UA_ModifyMonitoredItemsRequest_clear(&modifiedRequest);
     return response;
 }
@@ -985,10 +990,10 @@ UA_Client_MonitoredItems_modify_async(UA_Client *client,
                                       const UA_ModifyMonitoredItemsRequest request,
                                       UA_ClientAsyncServiceCallback callback,
                                       void *userdata, UA_UInt32 *requestId) {
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
     UA_Client_Subscription *sub = findSubscription(client, request.subscriptionId);
     if(!sub) {
-        UA_UNLOCK(&client->clientMutex);
+        unlockClient(client);
         return UA_STATUSCODE_BADSUBSCRIPTIONIDINVALID;
     }
 
@@ -1000,7 +1005,7 @@ UA_Client_MonitoredItems_modify_async(UA_Client *client,
         client, &modifiedRequest, &UA_TYPES[UA_TYPES_MODIFYMONITOREDITEMSREQUEST],
         callback, &UA_TYPES[UA_TYPES_MODIFYMONITOREDITEMSRESPONSE], userdata, requestId);
 
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
     UA_ModifyMonitoredItemsRequest_clear(&modifiedRequest);
     return statusCode;
 }
@@ -1083,9 +1088,7 @@ processDataChangeNotification(UA_Client *client, UA_Client_Subscription *sub,
             void *monC = mon->context;
             UA_UInt32 subId = sub->subscriptionId;
             UA_UInt32 monId = mon->monitoredItemId;
-            UA_UNLOCK(&client->clientMutex);
             mon->handler.dataChangeCallback(client, subId, subC, monId, monC, &min->value);
-            UA_LOCK(&client->clientMutex);
         }
     }
 }
@@ -1123,11 +1126,9 @@ processEventNotification(UA_Client *client, UA_Client_Subscription *sub,
         void *monC = mon->context;
         UA_UInt32 subId = sub->subscriptionId;
         UA_UInt32 monId = mon->monitoredItemId;
-        UA_UNLOCK(&client->clientMutex);
         mon->handler.eventCallback(client, subId, subC, monId, monC,
                                    eventFieldList->eventFieldsSize,
                                    eventFieldList->eventFields);
-        UA_LOCK(&client->clientMutex);
     }
 }
 
@@ -1160,10 +1161,8 @@ processNotificationMessage(UA_Client *client, UA_Client_Subscription *sub,
         if(sub->statusChangeCallback) {
             void *subC = sub->context;
             UA_UInt32 subId = sub->subscriptionId;
-            UA_UNLOCK(&client->clientMutex);
             sub->statusChangeCallback(client, subId, subC,
                                       (UA_StatusChangeNotification*)msg->content.decoded.data);
-            UA_LOCK(&client->clientMutex);
         } else {
             UA_LOG_WARNING(client->config.logging, UA_LOGCATEGORY_CLIENT,
                            "Dropped a StatusChangeNotification since no "
@@ -1239,9 +1238,7 @@ __Client_Subscriptions_processPublishResponse(UA_Client *client, UA_PublishReque
         if(client->config.subscriptionInactivityCallback) {
             void *subC = sub->context;
             UA_UInt32 subId = sub->subscriptionId;
-            UA_UNLOCK(&client->clientMutex);
             client->config.subscriptionInactivityCallback(client, subId, subC);
-            UA_LOCK(&client->clientMutex);
         }
         UA_LOG_WARNING(client->config.logging, UA_LOGCATEGORY_CLIENT,
                        "Received Timeout for Publish Response");
@@ -1305,7 +1302,7 @@ processPublishResponseAsync(UA_Client *client, void *userdata,
     UA_PublishRequest *req = (UA_PublishRequest*)userdata;
     UA_PublishResponse *res = (UA_PublishResponse*)response;
 
-    UA_LOCK(&client->clientMutex);
+    lockClient(client);
 
     /* Process the response */
     __Client_Subscriptions_processPublishResponse(client, req, res);
@@ -1316,7 +1313,7 @@ processPublishResponseAsync(UA_Client *client, void *userdata,
     /* Fill up the outstanding publish requests */
     __Client_Subscriptions_backgroundPublish(client);
 
-    UA_UNLOCK(&client->clientMutex);
+    unlockClient(client);
 }
 
 void
@@ -1359,9 +1356,7 @@ __Client_Subscriptions_backgroundPublishInactivityCheck(UA_Client *client) {
             if(client->config.subscriptionInactivityCallback) {
                 void *subC = sub->context;
                 UA_UInt32 subId = sub->subscriptionId;
-                UA_UNLOCK(&client->clientMutex);
                 client->config.subscriptionInactivityCallback(client, subId, subC);
-                UA_LOCK(&client->clientMutex);
             }
             UA_LOG_WARNING(client->config.logging, UA_LOGCATEGORY_CLIENT,
                            "Inactivity for Subscription %" PRIu32 ".", sub->subscriptionId);
