@@ -87,9 +87,9 @@ UA_UInt16 UA_Server_addNamespace(UA_Server *server, const char* name) {
     UA_String nameString;
     nameString.length = strlen(name);
     nameString.data = (UA_Byte*)(uintptr_t)name;
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_UInt16 retVal = addNamespace(server, nameString);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return retVal;
 }
 
@@ -130,18 +130,18 @@ getNamespaceByIndex(UA_Server *server, const size_t namespaceIndex,
 UA_StatusCode
 UA_Server_getNamespaceByName(UA_Server *server, const UA_String namespaceUri,
                              size_t *foundIndex) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_StatusCode res = getNamespaceByName(server, namespaceUri, foundIndex);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return res;
 }
 
 UA_StatusCode
 UA_Server_getNamespaceByIndex(UA_Server *server, const size_t namespaceIndex,
                               UA_String *foundUri) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_StatusCode res = getNamespaceByIndex(server, namespaceIndex, foundUri);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return res;
 }
 
@@ -253,7 +253,7 @@ UA_Server_delete(UA_Server *server) {
         return UA_STATUSCODE_BADINTERNALERROR;
     }
 
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     session_list_entry *current, *temp;
     LIST_FOREACH_SAFE(current, &server->sessions, pointers, temp) {
@@ -297,7 +297,7 @@ UA_Server_delete(UA_Server *server) {
     ZIP_ITER(UA_ServerComponentTree, &server->serverComponents,
              removeServerComponent, server);
 
-    UA_UNLOCK(&server->serviceMutex); /* The timer has its own mutex */
+    unlockServer(server); /* The timer has its own mutex */
 
     /* Clean up the config */
     UA_ServerConfig_clean(&server->config);
@@ -315,10 +315,10 @@ UA_Server_delete(UA_Server *server) {
  * sessions. */
 static void
 serverHouseKeeping(UA_Server *server, void *_) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_DateTime nowMonotonic = UA_DateTime_nowMonotonic();
     UA_Server_cleanupSessions(server, nowMonotonic);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
 }
 
 /********************/
@@ -348,7 +348,7 @@ UA_Server_init(UA_Server *server) {
 #endif
 
     UA_LOCK_INIT(&server->serviceMutex);
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     /* Initialize the adminSession */
     UA_Session_init(&server->adminSession);
@@ -387,9 +387,7 @@ UA_Server_init(UA_Server *server) {
     UA_CHECK_STATUS(res, goto cleanup);
 
 #ifdef UA_ENABLE_NODESET_INJECTOR
-    UA_UNLOCK(&server->serviceMutex);
     res = UA_Server_injectNodesets(server);
-    UA_LOCK(&server->serviceMutex);
     UA_CHECK_STATUS(res, goto cleanup);
 #endif
 
@@ -409,11 +407,11 @@ UA_Server_init(UA_Server *server) {
 #endif /* UA_ENABLE_PUBSUB_MONITORING */
 #endif /* UA_ENABLE_PUBSUB */
 
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return server;
 
  cleanup:
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     UA_Server_delete(server);
     return NULL;
 }
@@ -461,11 +459,11 @@ setServerShutdown(UA_Server *server) {
 UA_StatusCode
 UA_Server_addTimedCallback(UA_Server *server, UA_ServerCallback callback,
                            void *data, UA_DateTime date, UA_UInt64 *callbackId) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_StatusCode retval = server->config.eventLoop->
         addTimedCallback(server->config.eventLoop, (UA_Callback)callback,
                          server, data, date, callbackId);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return retval;
 }
 
@@ -483,9 +481,9 @@ UA_StatusCode
 UA_Server_addRepeatedCallback(UA_Server *server, UA_ServerCallback callback,
                               void *data, UA_Double interval_ms,
                               UA_UInt64 *callbackId) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_StatusCode res = addRepeatedCallback(server, callback, data, interval_ms, callbackId);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return res;
 }
 
@@ -501,10 +499,10 @@ changeRepeatedCallbackInterval(UA_Server *server, UA_UInt64 callbackId,
 UA_StatusCode
 UA_Server_changeRepeatedCallbackInterval(UA_Server *server, UA_UInt64 callbackId,
                                          UA_Double interval_ms) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_StatusCode retval =
         changeRepeatedCallbackInterval(server, callbackId, interval_ms);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return retval;
 }
 
@@ -519,9 +517,9 @@ removeCallback(UA_Server *server, UA_UInt64 callbackId) {
 
 void
 UA_Server_removeCallback(UA_Server *server, UA_UInt64 callbackId) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     removeCallback(server, callbackId);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
 }
 
 static void
@@ -544,19 +542,16 @@ UA_Server_updateCertificate(UA_Server *server,
     UA_CHECK(server && oldCertificate && newCertificate && newPrivateKey,
              return UA_STATUSCODE_BADINTERNALERROR);
 
+    lockServer(server);
+
     if(closeSessions) {
         session_list_entry *current;
         LIST_FOREACH(current, &server->sessions, pointers) {
             UA_SessionHeader *header = &current->session.header;
-            if(UA_ByteString_equal(oldCertificate,
-                                    &header->channel->securityPolicy->localCertificate)) {
-                UA_LOCK(&server->serviceMutex);
+            if(UA_ByteString_equal(oldCertificate, &header->channel->securityPolicy->localCertificate))
                 UA_Server_removeSessionByToken(server, &header->authenticationToken,
                                                UA_SHUTDOWNREASON_CLOSE);
-                UA_UNLOCK(&server->serviceMutex);
-            }
         }
-
     }
 
     /* Gracefully close all SecureChannels. And restart the
@@ -571,6 +566,7 @@ UA_Server_updateCertificate(UA_Server *server,
     }
 
     size_t i = 0;
+    UA_StatusCode res = UA_STATUSCODE_GOOD;
     while(i < server->config.endpointsSize) {
         UA_EndpointDescription *ed = &server->config.endpoints[i];
         if(UA_ByteString_equal(&ed->serverCertificate, oldCertificate)) {
@@ -578,13 +574,18 @@ UA_Server_updateCertificate(UA_Server *server,
             UA_String_copy(newCertificate, &ed->serverCertificate);
             UA_SecurityPolicy *sp = getSecurityPolicyByUri(server,
                             &server->config.endpoints[i].securityPolicyUri);
-            UA_CHECK_MEM(sp, return UA_STATUSCODE_BADINTERNALERROR);
+            if(!sp) {
+                res = UA_STATUSCODE_BADINTERNALERROR;
+                break;
+            }
             sp->updateCertificateAndPrivateKey(sp, *newCertificate, *newPrivateKey);
         }
         i++;
     }
 
-    return UA_STATUSCODE_GOOD;
+    unlockServer(server);
+
+    return res;
 }
 
 /***************************/
@@ -646,20 +647,16 @@ UA_Server_getStatistics(UA_Server *server) {
 /* Main Server Loop */
 /********************/
 
-#define UA_MAXTIMEOUT 200 /* Max timeout in ms between main-loop iterations */
+#define UA_MAXTIMEOUT 500 /* Max timeout in ms between main-loop iterations */
 
 void
 setServerLifecycleState(UA_Server *server, UA_LifecycleState state) {
     UA_LOCK_ASSERT(&server->serviceMutex, 1);
-
     if(server->state == state)
         return;
     server->state = state;
-    if(server->config.notifyLifecycleState) {
-        UA_UNLOCK(&server->serviceMutex);
+    if(server->config.notifyLifecycleState)
         server->config.notifyLifecycleState(server, server->state);
-        UA_LOCK(&server->serviceMutex);
-    }
 }
 
 UA_LifecycleState
@@ -724,12 +721,12 @@ UA_Server_run_startup(UA_Server *server) {
     }
 
     /* Take the server lock */
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     /* Does the ApplicationURI match the local certificates? */
 #ifdef UA_ENABLE_ENCRYPTION
     retVal = verifyServerApplicationURI(server);
-    UA_CHECK_STATUS(retVal, UA_UNLOCK(&server->serviceMutex); return retVal);
+    UA_CHECK_STATUS(retVal, unlockServer(server); return retVal);
 #endif
 
     /* Are there enough SecureChannels possible for the max number of sessions? */
@@ -743,7 +740,7 @@ UA_Server_run_startup(UA_Server *server) {
     /* Add a regular callback for housekeeping tasks. With a 1s interval. */
     retVal = addRepeatedCallback(server, serverHouseKeeping,
                                  NULL, 1000.0, &server->houseKeepingCallbackId);
-    UA_CHECK_STATUS_ERROR(retVal, UA_UNLOCK(&server->serviceMutex); return retVal,
+    UA_CHECK_STATUS_ERROR(retVal, unlockServer(server); return retVal,
                           config->logging, UA_LOGCATEGORY_SERVER,
                           "Could not create the server housekeeping task");
 
@@ -793,7 +790,7 @@ UA_Server_run_startup(UA_Server *server) {
         /* Stop all server components that have already been started */
         ZIP_ITER(UA_ServerComponentTree, &server->serverComponents,
                  stopServerComponent, server);
-        UA_UNLOCK(&server->serviceMutex);
+        unlockServer(server);
         return UA_STATUSCODE_BADINTERNALERROR;
     }
 
@@ -801,7 +798,7 @@ UA_Server_run_startup(UA_Server *server) {
      * UA_Server_run_shutdown(server) to stop the server. */
     setServerLifecycleState(server, UA_LIFECYCLESTATE_STARTED);
 
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return UA_STATUSCODE_GOOD;
 }
 
@@ -848,12 +845,12 @@ UA_Server_run_shutdown(UA_Server *server) {
     if(server == NULL)
         return UA_STATUSCODE_BADINVALIDARGUMENT;
 
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     if(server->state != UA_LIFECYCLESTATE_STARTED) {
         UA_LOG_ERROR(server->config.logging, UA_LOGCATEGORY_SERVER,
                      "The server is not started, cannot be shut down");
-        UA_UNLOCK(&server->serviceMutex);
+        unlockServer(server);
         return UA_STATUSCODE_BADINTERNALERROR;
     }
 
@@ -882,7 +879,7 @@ UA_Server_run_shutdown(UA_Server *server) {
 
     /* Only stop the EventLoop if it is coupled to the server lifecycle  */
     if(server->config.externalEventLoop) {
-        UA_UNLOCK(&server->serviceMutex);
+        unlockServer(server);
         return UA_STATUSCODE_GOOD;
     }
 
@@ -891,9 +888,7 @@ UA_Server_run_shutdown(UA_Server *server) {
     UA_EventLoop *el = server->config.eventLoop;
     while(!testStoppedCondition(server) &&
           res == UA_STATUSCODE_GOOD) {
-        UA_UNLOCK(&server->serviceMutex);
         res = el->run(el, 100);
-        UA_LOCK(&server->serviceMutex);
     }
 
     /* Stop the EventLoop. Iterate until stopped. */
@@ -901,15 +896,13 @@ UA_Server_run_shutdown(UA_Server *server) {
     while(el->state != UA_EVENTLOOPSTATE_STOPPED &&
           el->state != UA_EVENTLOOPSTATE_FRESH &&
           res == UA_STATUSCODE_GOOD) {
-        UA_UNLOCK(&server->serviceMutex);
         res = el->run(el, 100);
-        UA_LOCK(&server->serviceMutex);
     }
 
     /* Set server lifecycle state to stopped if not already the case */
     setServerLifecycleState(server, UA_LIFECYCLESTATE_STOPPED);
 
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return res;
 }
 
@@ -928,3 +921,14 @@ UA_Server_run(UA_Server *server, const volatile UA_Boolean *running) {
     return UA_Server_run_shutdown(server);
 }
 
+void lockServer(UA_Server *server) {
+    if(UA_LIKELY(server->config.eventLoop && server->config.eventLoop->lock))
+        server->config.eventLoop->lock(server->config.eventLoop);
+    UA_LOCK(&server->serviceMutex);
+}
+
+void unlockServer(UA_Server *server) {
+    if(UA_LIKELY(server->config.eventLoop && server->config.eventLoop->unlock))
+        server->config.eventLoop->unlock(server->config.eventLoop);
+    UA_UNLOCK(&server->serviceMutex);
+}
