@@ -161,9 +161,9 @@ checkTimeouts(UA_Server *server, void *_) {
     UA_UNLOCK(&am->queueLock);
 
     /* Integrate async results and send out complete responses */
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     processAsyncResults(server);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
 }
 
 void
@@ -176,9 +176,9 @@ UA_AsyncManager_init(UA_AsyncManager *am, UA_Server *server) {
     UA_LOCK_INIT(&am->queueLock);
 
     /* Add a regular callback for cleanup and sending finished responses at a
-     * 100s interval. */
+     * 1s interval. */
     addRepeatedCallback(server, (UA_ServerCallback)checkTimeouts,
-                        NULL, 100.0, &am->checkTimeoutCallbackId);
+                        NULL, 1000.0, &am->checkTimeoutCallbackId);
 }
 
 void
@@ -395,11 +395,11 @@ setMethodNodeAsync(UA_Server *server, UA_Session *session,
 UA_StatusCode
 UA_Server_setMethodNodeAsync(UA_Server *server, const UA_NodeId id,
                              UA_Boolean isAsync) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     UA_StatusCode res =
         UA_Server_editNode(server, &server->adminSession, &id,
                            (UA_EditNodeCallback)setMethodNodeAsync, &isAsync);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return res;
 }
 
