@@ -715,7 +715,7 @@ processMSGDecoded(UA_Server *server, UA_SecureChannel *channel, UA_UInt32 reques
     UA_StatusCode channelRes = UA_STATUSCODE_GOOD;
     UA_ResponseHeader *rh = &response->responseHeader;
 
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     /* If it is an unencrypted (#None) channel, only allow the discovery services */
     if(server->config.securityPolicyNoneDiscoveryOnly &&
@@ -807,7 +807,7 @@ processMSGDecoded(UA_Server *server, UA_SecureChannel *channel, UA_UInt32 reques
             Service_Publish(server, session, &request->publishRequest, requestId);
 
         /* Don't send a response */
-        UA_UNLOCK(&server->serviceMutex);
+        unlockServer(server);
         goto update_statistics;
     }
 #endif
@@ -824,7 +824,7 @@ processMSGDecoded(UA_Server *server, UA_SecureChannel *channel, UA_UInt32 reques
          * statistic. */
         if(UA_LIKELY(finished))
             goto send_response;
-        UA_UNLOCK(&server->serviceMutex);
+        unlockServer(server);
         goto update_statistics;
     }
 #endif
@@ -834,7 +834,7 @@ processMSGDecoded(UA_Server *server, UA_SecureChannel *channel, UA_UInt32 reques
 
     /* Upon success, send the response. Otherwise a ServiceFault. */
  send_response:
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     channelRes = sendResponse(server, session, channel,
                               requestId, response, responseType);
 
@@ -952,11 +952,9 @@ processMSG(UA_Server *server, UA_SecureChannel *channel,
 
 /* Takes decoded messages starting at the nodeid of the content type. */
 static UA_StatusCode
-processSecureChannelMessage(void *application, UA_SecureChannel *channel,
+processSecureChannelMessage(UA_Server *server, UA_SecureChannel *channel,
                             UA_MessageType messagetype, UA_UInt32 requestId,
                             UA_ByteString *message) {
-    UA_Server *server = (UA_Server*)application;
-
     UA_StatusCode retval = UA_STATUSCODE_GOOD;
     switch(messagetype) {
     case UA_MESSAGETYPE_HEL:
@@ -1034,9 +1032,12 @@ purgeFirstChannelWithoutSession(UA_BinaryProtocolManager *bpm) {
 static UA_StatusCode
 configServerSecureChannel(void *application, UA_SecureChannel *channel,
                           const UA_AsymmetricAlgorithmSecurityHeader *asymHeader) {
+    if(channel->securityPolicy)
+        return UA_STATUSCODE_GOOD;
+
     /* Iterate over available endpoints and choose the correct one */
+    UA_Server *server = (UA_Server *)application;
     UA_SecurityPolicy *securityPolicy = NULL;
-    UA_Server *const server = (UA_Server *const) application;
     for(size_t i = 0; i < server->config.securityPoliciesSize; ++i) {
         UA_SecurityPolicy *policy = &server->config.securityPolicies[i];
         if(!UA_ByteString_equal(&asymHeader->securityPolicyUri, &policy->policyUri))
@@ -1106,6 +1107,7 @@ createServerSecureChannel(UA_BinaryProtocolManager *bpm, UA_ConnectionManager *c
     entry->channel.config = connConfig;
     entry->channel.certificateVerification = &config->secureChannelPKI;
     entry->channel.processOPNHeader = configServerSecureChannel;
+    entry->channel.processOPNHeaderApplication = server;
     entry->channel.connectionManager = cm;
     entry->channel.connectionId = connectionId;
 
@@ -1258,17 +1260,15 @@ serverNetworkCallback(UA_ConnectionManager *cm, uintptr_t connectionId,
             return;
         }
 
-        UA_LOG_INFO_CHANNEL(bpm->logging, channel, "SecureChannel created");
-
         /* Set the new channel as the new context for the connection */
         *connectionContext = (void*)channel;
-        return;
-    }
 
-    /* The connection has fully opened */
-    if(channel->state < UA_SECURECHANNELSTATE_CONNECTED)
+        /* Set the channel state to CONNECTED until the HEL message is received */
         channel->state = UA_SECURECHANNELSTATE_CONNECTED;
 
+        UA_LOG_INFO_CHANNEL(bpm->logging, channel, "SecureChannel created");
+    }
+
     /* Received a message on a normal connection */
 #ifdef UA_DEBUG_DUMP_PKGS
     UA_dump_hex_pkg(message->data, message->length);
@@ -1277,8 +1277,24 @@ serverNetworkCallback(UA_ConnectionManager *cm, uintptr_t connectionId,
     UA_debug_dumpCompleteChunk(server, channel->connection, message);
 #endif
 
-    retval = UA_SecureChannel_processBuffer(channel, bpm->server,
-                                            processSecureChannelMessage, &msg);
+    /* Process all complete messages */
+    retval = UA_SecureChannel_loadBuffer(channel, msg);
+    while(UA_LIKELY(retval == UA_STATUSCODE_GOOD)) {
+        UA_MessageType messageType;
+        UA_UInt32 requestId = 0;
+        UA_ByteString payload = UA_BYTESTRING_NULL;
+        UA_Boolean copied = false;
+        retval = UA_SecureChannel_getCompleteMessage(channel, &messageType, &requestId,
+                                                     &payload, &copied);
+        if(retval != UA_STATUSCODE_GOOD || payload.length == 0)
+            break;
+        retval = processSecureChannelMessage(bpm->server, channel,
+                                             messageType, requestId, &payload);
+        if(copied)
+            UA_ByteString_clear(&payload);
+    }
+    retval |= UA_SecureChannel_persistBuffer(channel);
+
     if(retval != UA_STATUSCODE_GOOD) {
         UA_LOG_WARNING_CHANNEL(bpm->logging, channel,
                                "Processing the message failed with error %s",
@@ -1357,7 +1373,7 @@ createServerConnection(UA_BinaryProtocolManager *bpm, const UA_String *serverUrl
 static void
 secureChannelHouseKeeping(UA_Server *server, void *context) {
     UA_BinaryProtocolManager *bpm = (UA_BinaryProtocolManager*)context;
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     UA_DateTime nowMonotonic = UA_DateTime_nowMonotonic();
     channel_entry *entry;
@@ -1393,7 +1409,7 @@ secureChannelHouseKeeping(UA_Server *server, void *context) {
             UA_SecureChannel_shutdown(&entry->channel, UA_SHUTDOWNREASON_TIMEOUT);
         }
     }
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
 }
 
 /**********************/
@@ -1453,7 +1469,7 @@ sendRHEMessage(UA_Server *server, uintptr_t connectionId,
 
 static void
 retryReverseConnectCallback(UA_Server *server, void *context) {
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     UA_BinaryProtocolManager *bpm = (UA_BinaryProtocolManager*)context;
 
@@ -1467,7 +1483,7 @@ retryReverseConnectCallback(UA_Server *server, void *context) {
         attemptReverseConnect(bpm, rc);
     }
 
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
 }
 
 UA_StatusCode
@@ -1591,7 +1607,7 @@ UA_Server_addReverseConnect(UA_Server *server, UA_String url,
     newContext->stateCallback = stateCallback;
     newContext->callbackContext = callbackContext;
 
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     /* Register the retry callback */
     setReverseConnectRetryCallback(bpm, true);
@@ -1605,7 +1621,7 @@ UA_Server_addReverseConnect(UA_Server *server, UA_String url,
     /* Attempt to connect right away */
     res = attemptReverseConnect(bpm, newContext);
 
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return res;
 }
 
@@ -1613,7 +1629,7 @@ UA_StatusCode
 UA_Server_removeReverseConnect(UA_Server *server, UA_UInt64 handle) {
     UA_StatusCode result = UA_STATUSCODE_BADNOTFOUND;
 
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
 
     UA_ServerComponent *sc =
         getServerComponentByName(server, UA_STRING("binary"));
@@ -1621,7 +1637,7 @@ UA_Server_removeReverseConnect(UA_Server *server, UA_UInt64 handle) {
     if(!bpm) {
         UA_LOG_ERROR(server->config.logging, UA_LOGCATEGORY_SERVER,
                      "No BinaryProtocolManager configured");
-        UA_UNLOCK(&server->serviceMutex);
+        unlockServer(server);
         return UA_STATUSCODE_BADINTERNALERROR;
     }
 
@@ -1649,8 +1665,7 @@ UA_Server_removeReverseConnect(UA_Server *server, UA_UInt64 handle) {
     if(LIST_EMPTY(&bpm->reverseConnects))
         setReverseConnectRetryCallback(bpm, false);
 
-    UA_UNLOCK(&server->serviceMutex);
-
+    unlockServer(server);
     return result;
 }
 
@@ -1744,8 +1759,23 @@ serverReverseConnectCallback(UA_ConnectionManager *cm, uintptr_t connectionId,
 
     /* The connection is fully opened and we have a SecureChannel.
      * Process the received buffer */
-    retval = UA_SecureChannel_processBuffer(context->channel, bpm->server,
-                                            processSecureChannelMessage, &msg);
+    retval = UA_SecureChannel_loadBuffer(context->channel, msg);
+    while(UA_LIKELY(retval == UA_STATUSCODE_GOOD)) {
+        UA_MessageType messageType;
+        UA_UInt32 requestId = 0;
+        UA_ByteString payload = UA_BYTESTRING_NULL;
+        UA_Boolean copied = false;
+        retval = UA_SecureChannel_getCompleteMessage(context->channel, &messageType,
+                                                     &requestId, &payload, &copied);
+        if(retval != UA_STATUSCODE_GOOD || payload.length == 0)
+            break;
+        retval = processSecureChannelMessage(bpm->server, context->channel,
+                                             messageType, requestId, &payload);
+        if(copied)
+            UA_ByteString_clear(&payload);
+    }
+    retval |= UA_SecureChannel_persistBuffer(context->channel);
+
     if(retval != UA_STATUSCODE_GOOD) {
         UA_LOG_WARNING_CHANNEL(bpm->logging, context->channel,
                                "Processing the message failed with error %s",
@@ -1758,7 +1788,6 @@ serverReverseConnectCallback(UA_ConnectionManager *cm, uintptr_t connectionId,
         error.reason = UA_STRING_NULL;
         UA_SecureChannel_sendError(context->channel, &error);
         UA_SecureChannel_shutdown(context->channel, UA_SHUTDOWNREASON_ABORT);
-
         setReverseConnectState(bpm->server, context, UA_SECURECHANNELSTATE_CLOSING);
         return;
     }
