@@ -266,13 +266,11 @@ callWithMethodAndObject(UA_Server *server, UA_Session *session,
     /* Verify access rights */
     UA_Boolean executable = method->executable;
     if(session != &server->adminSession) {
-        UA_UNLOCK(&server->serviceMutex);
         executable = executable && server->config.accessControl.
             getUserExecutableOnObject(server, &server->config.accessControl,
                                       &session->sessionId, session->sessionHandle,
                                       &request->methodId, method->head.context,
                                       &request->objectId, object->head.context);
-        UA_LOCK(&server->serviceMutex);
     }
 
     if(!executable) {
@@ -348,14 +346,22 @@ callWithMethodAndObject(UA_Server *server, UA_Session *session,
     /* Release the output arguments node */
     UA_NODESTORE_RELEASE(server, (const UA_Node*)outputArguments);
 
-    /* Call the method */
-    UA_UNLOCK(&server->serviceMutex);
+    /* Call the method. If this is an async method, unlock the server lock for
+     * the duration of the (long-running) call. */
+#if UA_MULTITHREADING >= 100
+    if(method->async)
+        unlockServer(server);
+#endif
     result->statusCode = method->method(server, &session->sessionId, session->sessionHandle,
                                         &method->head.nodeId, method->head.context,
                                         &object->head.nodeId, object->head.context,
                                         request->inputArgumentsSize, mutableInputArgs,
                                         result->outputArgumentsSize, result->outputArguments);
-    UA_LOCK(&server->serviceMutex);
+#if UA_MULTITHREADING >= 100
+    if(method->async)
+        lockServer(server);
+#endif
+
     /* TODO: Verify Output matches the argument definition */
 }
 
@@ -523,9 +529,9 @@ UA_CallMethodResult
 UA_Server_call(UA_Server *server, const UA_CallMethodRequest *request) {
     UA_CallMethodResult result;
     UA_CallMethodResult_init(&result);
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     Operation_CallMethod(server, &server->adminSession, NULL, request, &result);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     return result;
 }
 
