@@ -938,9 +938,7 @@ ENCODE_JSON(ExtensionObject) {
         return writeChars(ctx, "null", 4);
 
     /* Must have a type set if data is decoded */
-    if(src->encoding != UA_EXTENSIONOBJECT_ENCODED_BYTESTRING &&
-       src->encoding != UA_EXTENSIONOBJECT_ENCODED_XML &&
-       !src->content.decoded.type)
+    if(src->encoding >= UA_EXTENSIONOBJECT_DECODED && !src->content.decoded.type)
         return UA_STATUSCODE_BADENCODINGERROR;
 
     status ret = writeJsonObjStart(ctx);
@@ -1101,7 +1099,7 @@ ENCODE_JSON(Variant) {
             if(hasDimensions) {
                 ret |= writeJsonKey(ctx, UA_JSONKEY_DIMENSION);
                 ret |= encodeJsonArray(ctx, src->arrayDimensions, src->arrayDimensionsSize,
-                                       &UA_TYPES[UA_TYPES_INT32]);
+                                       &UA_TYPES[UA_TYPES_UINT32]);
             }
         } else {
             /* Special case of non-reversible array with dimensions */
@@ -2156,7 +2154,7 @@ getExtensionObjectType(ParseCtx *ctx) {
         return NULL;
 
     size_t oldIndex = ctx->index;
-    ctx->index = (UA_UInt16)typeIdIndex;
+    ctx->index = typeIdIndex;
 
     /* Decode the type NodeId */
     UA_NodeId typeId;
