@@ -182,7 +182,7 @@ isTrue(uint8_t expr) {
  * ----------------- */
 
 #ifdef UA_ENABLE_DISCOVERY_SEMAPHORE
-# ifdef _WIN32
+# ifdef UA_ARCHITECTURE_WIN32
 #  include <io.h>
 #  define UA_fileExists(X) ( _access(X, 0) == 0)
 # else
