@@ -13,6 +13,8 @@
 #include <open62541/server.h>
 #include <open62541/server_config_default.h>
 
+#include <stdlib.h>
+
 #include "certificates.h"
 #include "check.h"
 #include "thread_wrapper.h"
@@ -67,7 +69,6 @@ static void setup1(void) {
     issuerList[0] = intermediateCa;
     issuerList[1] = rootCa;
 
-    /* Loading of a revocation list currently unsupported */
     size_t revocationListSize = 2;
     UA_STACKARRAY(UA_ByteString, revocationList, revocationListSize);
     revocationList[0] = rootCaCrl;
@@ -130,7 +131,6 @@ static void setup2(void) {
     issuerList[0] = intermediateCa;
     issuerList[1] = rootCa;
 
-    /* Loading of a revocation list currently unsupported */
     size_t revocationListSize = 2;
     UA_STACKARRAY(UA_ByteString, revocationList, revocationListSize);
     revocationList[0] = rootCaCrl;
@@ -193,7 +193,6 @@ static void setup3(void) {
     issuerList[0] = rootCa;
     issuerList[1] = intermediateCa;
 
-    /* Loading of a revocation list currently unsupported */
     size_t revocationListSize = 2;
     UA_STACKARRAY(UA_ByteString, revocationList, revocationListSize);
     revocationList[0] = rootCaCrl;
