@@ -5,7 +5,6 @@
  * Copyright (c) 2019 Fraunhofer IOSB (Author: Andreas Ebner)
  */
 
-#include <open62541/plugin/pubsub_udp.h>
 #include <open62541/server.h>
 #include <open62541/server_config_default.h>
 
@@ -15,10 +14,14 @@
 
 #include <check.h>
 #include <stdio.h>
+#include <stdlib.h>
 
+UA_EventLoop *rtEventLoop = NULL;
 UA_Server *server = NULL;
 UA_NodeId connectionIdentifier, publishedDataSetIdent, writerGroupIdent, dataSetWriterIdent, dataSetFieldIdent;
 
+UA_DataValue *staticSource1, *staticSource2;
+
 static UA_StatusCode
 addMinimalPubSubConfiguration(void){
     UA_StatusCode retVal = UA_STATUSCODE_GOOD;
@@ -28,9 +31,11 @@ addMinimalPubSubConfiguration(void){
     connectionConfig.name = UA_STRING("UDP-UADP Connection 1");
     connectionConfig.transportProfileUri = UA_STRING("http://opcfoundation.org/UA-Profile/Transport/pubsub-udp-uadp");
     connectionConfig.enabled = UA_TRUE;
+    connectionConfig.eventLoop = rtEventLoop;
     UA_NetworkAddressUrlDataType networkAddressUrl = {UA_STRING_NULL , UA_STRING("opc.udp://224.0.0.22:4840/")};
     UA_Variant_setScalar(&connectionConfig.address, &networkAddressUrl, &UA_TYPES[UA_TYPES_NETWORKADDRESSURLDATATYPE]);
-    connectionConfig.publisherId.numeric = UA_UInt32_random();
+    connectionConfig.publisherIdType = UA_PUBLISHERIDTYPE_UINT32;
+    connectionConfig.publisherId.uint32 = UA_UInt32_random();
     retVal = UA_Server_addPubSubConnection(server, &connectionConfig, &connectionIdentifier);
     if(retVal != UA_STATUSCODE_GOOD)
         return retVal;
@@ -46,66 +51,58 @@ addMinimalPubSubConfiguration(void){
 
 static void setup(void) {
     server = UA_Server_new();
+    ck_assert(server != NULL);
     UA_ServerConfig *config = UA_Server_getConfig(server);
     UA_ServerConfig_setDefault(config);
-    UA_ServerConfig_addPubSubTransportLayer(config, UA_PubSubTransportLayerUDPMP());
     UA_Server_run_startup(server);
+
+    rtEventLoop = UA_EventLoop_new_POSIX(server->config.logging);
+
+    /* Add the TCP connection manager */
+    UA_ConnectionManager *tcpCM =
+        UA_ConnectionManager_new_POSIX_TCP(UA_STRING("tcp connection manager"));
+    rtEventLoop->registerEventSource(rtEventLoop, (UA_EventSource *)tcpCM);
+
+    /* Add the UDP connection manager */
+    UA_ConnectionManager *udpCM =
+        UA_ConnectionManager_new_POSIX_UDP(UA_STRING("udp connection manager"));
+    rtEventLoop->registerEventSource(rtEventLoop, (UA_EventSource *)udpCM);
+
+    rtEventLoop->start(rtEventLoop);
 }
 
 static void teardown(void) {
     UA_Server_run_shutdown(server);
+
+    if(rtEventLoop->state != UA_EVENTLOOPSTATE_FRESH &&
+       rtEventLoop->state != UA_EVENTLOOPSTATE_STOPPED) {
+        rtEventLoop->stop(rtEventLoop);
+        while(rtEventLoop->state != UA_EVENTLOOPSTATE_STOPPED) {
+            rtEventLoop->run(rtEventLoop, 100);
+        }
+    }
+
     UA_Server_delete(server);
-}
+    rtEventLoop->logger = NULL; /* Don't access the logger that was removed with
+                                   the server */
+    rtEventLoop->free(rtEventLoop);
 
-typedef struct {
-    UA_ByteString *buffer;
-    size_t offset;
-} UA_ReceiveContext;
+    rtEventLoop = NULL;
+    server = NULL;
 
-static UA_StatusCode
-recvTestFun(UA_PubSubChannel *channel, void *context,
-               const UA_ByteString *buffer) {
-    UA_ReceiveContext *ctx = (UA_ReceiveContext*)context;
-    memcpy(ctx->buffer->data + ctx->offset, buffer->data, buffer->length);
-    ctx->offset += buffer->length;
-    ctx->buffer->length = ctx->offset;
-    return UA_STATUSCODE_GOOD;
-}
-
-static void
-receiveSingleMessage(UA_ByteString buffer, UA_PubSubConnection *connection,
-                     UA_NetworkMessage *networkMessage) {
-    if(UA_ByteString_allocBuffer(&buffer, 512) != UA_STATUSCODE_GOOD) {
-        ck_abort_msg("Message buffer allocation failed!");
-        return;
+    /* Clean these up only after the server is done */
+    if(staticSource1) {
+        UA_DataValue_delete(staticSource1);
+        staticSource1 = NULL;
     }
-
-    if(!connection->channel) {
-        ck_abort_msg("No connection established");
-        return;
+    if(staticSource2) {
+        UA_DataValue_delete(staticSource2);
+        staticSource2 = NULL;
     }
-
-    UA_ReceiveContext testCtx = {&buffer, 0};
-    UA_StatusCode retval =
-        connection->channel->receive(connection->channel, NULL,
-                                     recvTestFun, &testCtx, 1000000);
-    if(retval != UA_STATUSCODE_GOOD || buffer.length == 0) {
-        buffer.length = 512;
-        UA_ByteString_clear(&buffer);
-        ck_abort_msg("Expected message not received!");
-    }
-    memset(networkMessage, 0, sizeof(UA_NetworkMessage));
-    size_t currentPosition = 0;
-    UA_NetworkMessage_decodeBinary(&buffer, &currentPosition, networkMessage);
-    UA_ByteString_clear(&buffer);
 }
 
 START_TEST(PublishSingleFieldWithStaticValueSource) {
         ck_assert(addMinimalPubSubConfiguration() == UA_STATUSCODE_GOOD);
-        UA_PubSubConnection *connection = UA_PubSubConnection_findConnectionbyId(server, connectionIdentifier);
-        ck_assert(connection);
-        UA_StatusCode rv = connection->channel->regist(connection->channel, NULL, NULL);
-        ck_assert(rv == UA_STATUSCODE_GOOD);
         UA_WriterGroupConfig writerGroupConfig;
         memset(&writerGroupConfig, 0, sizeof(UA_WriterGroupConfig));
         writerGroupConfig.name = UA_STRING("Demo WriterGroup");
@@ -132,10 +129,10 @@ START_TEST(PublishSingleFieldWithStaticValueSource) {
         /* Create Variant and configure as DataSetField source */
         UA_UInt32 *intValue = UA_UInt32_new();
         *intValue = 1000;
-        UA_DataValue *dataValue = UA_DataValue_new();
-        UA_Variant_setScalar(&dataValue->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
+        staticSource1 = UA_DataValue_new();
+        UA_Variant_setScalar(&staticSource1->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
         dsfConfig.field.variable.rtValueSource.rtFieldSourceEnabled = UA_TRUE;
-        dsfConfig.field.variable.rtValueSource.staticValueSource = &dataValue;
+        dsfConfig.field.variable.rtValueSource.staticValueSource = &staticSource1;
         ck_assert(UA_Server_addDataSetField(server, publishedDataSetIdent, &dsfConfig, &dataSetFieldIdent).result == UA_STATUSCODE_GOOD);
 
         /* DataSetWriter muste be added AFTER addDataSetField otherwize lastSamples will be uninitialized */
@@ -143,21 +140,13 @@ START_TEST(PublishSingleFieldWithStaticValueSource) {
 
         ck_assert(UA_Server_freezeWriterGroupConfiguration(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        UA_ByteString buffer;
-        UA_ByteString_init(&buffer);
-        UA_NetworkMessage networkMessage;
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        ck_assert((*((UA_UInt32 *)networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data)) == 1000);
-        UA_NetworkMessage_clear(&networkMessage);
-        UA_DataValue_delete(dataValue);
-    } END_TEST
+
+        UA_Server_run_iterate(server, false);
+} END_TEST
 
 START_TEST(PublishSingleFieldWithDifferentBinarySizes) {
         ck_assert(addMinimalPubSubConfiguration() == UA_STATUSCODE_GOOD);
-        UA_PubSubConnection *connection = UA_PubSubConnection_findConnectionbyId(server, connectionIdentifier);
-        ck_assert(connection);
-        UA_StatusCode rv = connection->channel->regist(connection->channel, NULL, NULL);
-        ck_assert(rv == UA_STATUSCODE_GOOD);
+
         UA_WriterGroupConfig writerGroupConfig;
         memset(&writerGroupConfig, 0, sizeof(UA_WriterGroupConfig));
         writerGroupConfig.name = UA_STRING("Test WriterGroup");
@@ -172,7 +161,9 @@ START_TEST(PublishSingleFieldWithDifferentBinarySizes) {
         writerGroupConfig.messageSettings.content.decoded.type =
             &UA_TYPES[UA_TYPES_UADPWRITERGROUPMESSAGEDATATYPE];
         writerGroupConfig.messageSettings.encoding = UA_EXTENSIONOBJECT_DECODED;
-        ck_assert(UA_Server_addWriterGroup(server, connectionIdentifier, &writerGroupConfig, &writerGroupIdent) == UA_STATUSCODE_GOOD);
+        UA_StatusCode res = UA_Server_addWriterGroup(server, connectionIdentifier,
+                                                     &writerGroupConfig, &writerGroupIdent);
+        ck_assert(res == UA_STATUSCODE_GOOD);
         UA_UadpWriterGroupMessageDataType_delete(wgm);
         UA_DataSetWriterConfig dataSetWriterConfig;
         memset(&dataSetWriterConfig, 0, sizeof(UA_DataSetWriterConfig));
@@ -181,46 +172,23 @@ START_TEST(PublishSingleFieldWithDifferentBinarySizes) {
         UA_DataSetFieldConfig dsfConfig;
         memset(&dsfConfig, 0, sizeof(UA_DataSetFieldConfig));
         /* Create Variant and configure as DataSetField source */
-        UA_String stringValue = UA_STRING_ALLOC("12345");
-        UA_DataValue *dataValue = UA_DataValue_new();
-        UA_Variant_setScalar(&dataValue->value, &stringValue, &UA_TYPES[UA_TYPES_STRING]);
-        dataValue->value.storageType = UA_VARIANT_DATA_NODELETE;
+        UA_String stringValue = UA_STRING("12345");
+        staticSource1 = UA_DataValue_new();
+        UA_Variant_setScalar(&staticSource1->value, &stringValue, &UA_TYPES[UA_TYPES_STRING]);
+        staticSource1->value.storageType = UA_VARIANT_DATA_NODELETE;
         dsfConfig.field.variable.rtValueSource.rtFieldSourceEnabled = UA_TRUE;
-        dsfConfig.field.variable.rtValueSource.staticValueSource = &dataValue;
+        dsfConfig.field.variable.rtValueSource.staticValueSource = &staticSource1;
         dsfConfig.field.variable.publishParameters.attributeId = UA_ATTRIBUTEID_VALUE;
         ck_assert(UA_Server_addDataSetField(server, publishedDataSetIdent, &dsfConfig, &dataSetFieldIdent).result == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_addDataSetWriter(server, writerGroupIdent, publishedDataSetIdent, &dataSetWriterConfig, &dataSetWriterIdent) == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_freezeWriterGroupConfiguration(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        UA_ByteString buffer;
-        UA_ByteString_init(&buffer);
-        UA_NetworkMessage networkMessage;
-        memset(&networkMessage, 0, sizeof(networkMessage));
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        UA_String compareString = UA_STRING("12345");
-        ck_assert(UA_String_equal(((UA_String *) networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data), &compareString) == UA_TRUE);
-        UA_NetworkMessage_clear(&networkMessage);
-        compareString = UA_STRING("123456789");
-        stringValue.data = (UA_Byte *) UA_realloc(stringValue.data, 9);
-        stringValue.length = 9;
-        memcpy(stringValue.data, "123456789", 9);
-        UA_ByteString_init(&buffer);
-        memset(&networkMessage, 0, sizeof(networkMessage));
-        ck_assert(UA_Server_setWriterGroupDisabled(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        ck_assert(UA_String_equal(((UA_String *) networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data), &compareString) == UA_TRUE);
-        UA_NetworkMessage_clear(&networkMessage);
-        UA_String_clear(&stringValue);
-        UA_DataValue_delete(dataValue);
+
+        UA_Server_run_iterate(server, false);
     } END_TEST
 
 START_TEST(SetupInvalidPubSubConfigWithStaticValueSource) {
         ck_assert(addMinimalPubSubConfiguration() == UA_STATUSCODE_GOOD);
-        UA_PubSubConnection *connection = UA_PubSubConnection_findConnectionbyId(server, connectionIdentifier);
-        ck_assert(connection);
-        UA_StatusCode rv = connection->channel->regist(connection->channel, NULL, NULL);
-        ck_assert(rv == UA_STATUSCODE_GOOD);
         UA_WriterGroupConfig writerGroupConfig;
         memset(&writerGroupConfig, 0, sizeof(UA_WriterGroupConfig));
         writerGroupConfig.name = UA_STRING("Test WriterGroup");
@@ -257,10 +225,6 @@ START_TEST(SetupInvalidPubSubConfigWithStaticValueSource) {
 
 START_TEST(PublishSingleFieldWithFixedOffsets) {
         ck_assert(addMinimalPubSubConfiguration() == UA_STATUSCODE_GOOD);
-        UA_PubSubConnection *connection = UA_PubSubConnection_findConnectionbyId(server, connectionIdentifier);
-        ck_assert(connection);
-        UA_StatusCode rv = connection->channel->regist(connection->channel, NULL, NULL);
-        ck_assert(rv == UA_STATUSCODE_GOOD);
         UA_WriterGroupConfig writerGroupConfig;
         memset(&writerGroupConfig, 0, sizeof(UA_WriterGroupConfig));
         writerGroupConfig.name = UA_STRING("Demo WriterGroup");
@@ -286,10 +250,10 @@ START_TEST(PublishSingleFieldWithFixedOffsets) {
         // Create Variant and configure as DataSetField source
         UA_UInt32 *intValue = UA_UInt32_new();
         *intValue = (UA_UInt32) 1000;
-        UA_DataValue *dataValue = UA_DataValue_new();
-        UA_Variant_setScalar(&dataValue->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
+        staticSource1 = UA_DataValue_new();
+        UA_Variant_setScalar(&staticSource1->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
         dsfConfig.field.variable.rtValueSource.rtFieldSourceEnabled = UA_TRUE;
-        dsfConfig.field.variable.rtValueSource.staticValueSource = &dataValue;
+        dsfConfig.field.variable.rtValueSource.staticValueSource = &staticSource1;
         dsfConfig.field.variable.publishParameters.attributeId = UA_ATTRIBUTEID_VALUE;
         ck_assert(UA_Server_addDataSetField(server, publishedDataSetIdent, &dsfConfig, &dataSetFieldIdent).result == UA_STATUSCODE_GOOD);
 
@@ -297,24 +261,12 @@ START_TEST(PublishSingleFieldWithFixedOffsets) {
 
         ck_assert(UA_Server_freezeWriterGroupConfiguration(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        UA_ByteString buffer;
-        UA_ByteString_init(&buffer);
-        UA_NetworkMessage networkMessage;
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        ck_assert((*((UA_UInt32 *)networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data)) == 1000);
-        UA_NetworkMessage_clear(&networkMessage);
-        UA_DataValue_delete(dataValue);
 
-        UA_Server_run_shutdown(server);
-        UA_Server_delete(server);
-    } END_TEST
+        UA_Server_run_iterate(server, false);
+} END_TEST
 
 START_TEST(PublishPDSWithMultipleFieldsAndFixedOffset) {
         ck_assert(addMinimalPubSubConfiguration() == UA_STATUSCODE_GOOD);
-        UA_PubSubConnection *connection = UA_PubSubConnection_findConnectionbyId(server, connectionIdentifier);
-        ck_assert(connection);
-        UA_StatusCode rv = connection->channel->regist(connection->channel, NULL, NULL);
-        ck_assert(rv == UA_STATUSCODE_GOOD);
         UA_WriterGroupConfig writerGroupConfig;
         memset(&writerGroupConfig, 0, sizeof(UA_WriterGroupConfig));
         writerGroupConfig.name = UA_STRING("Demo WriterGroup");
@@ -340,18 +292,18 @@ START_TEST(PublishPDSWithMultipleFieldsAndFixedOffset) {
         // Create Variant and configure as DataSetField source
         UA_UInt32 *intValue = UA_UInt32_new();
         *intValue = (UA_UInt32) 1000;
-        UA_DataValue *dataValue = UA_DataValue_new();
-        UA_Variant_setScalar(&dataValue->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
+        staticSource1 = UA_DataValue_new();
+        UA_Variant_setScalar(&staticSource1->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
         dsfConfig.field.variable.rtValueSource.rtFieldSourceEnabled = UA_TRUE;
         dsfConfig.field.variable.rtValueSource.rtInformationModelNode = UA_FALSE;
-        dsfConfig.field.variable.rtValueSource.staticValueSource = &dataValue;
+        dsfConfig.field.variable.rtValueSource.staticValueSource = &staticSource1;
         dsfConfig.field.variable.publishParameters.attributeId = UA_ATTRIBUTEID_VALUE;
         ck_assert(UA_Server_addDataSetField(server, publishedDataSetIdent, &dsfConfig, NULL).result == UA_STATUSCODE_GOOD);
         UA_UInt32 *intValue2 = UA_UInt32_new();
         *intValue2 = (UA_UInt32) 2000;
-        UA_DataValue *dataValue2 = UA_DataValue_new();
-        UA_Variant_setScalar(&dataValue2->value, intValue2, &UA_TYPES[UA_TYPES_UINT32]);
-        dsfConfig.field.variable.rtValueSource.staticValueSource = &dataValue2;
+        staticSource2 = UA_DataValue_new();
+        UA_Variant_setScalar(&staticSource2->value, intValue2, &UA_TYPES[UA_TYPES_UINT32]);
+        dsfConfig.field.variable.rtValueSource.staticValueSource = &staticSource2;
         ck_assert(UA_Server_addDataSetField(server, publishedDataSetIdent, &dsfConfig, NULL).result == UA_STATUSCODE_GOOD);
 
         ck_assert(UA_Server_addDataSetWriter(server, writerGroupIdent, publishedDataSetIdent, &dataSetWriterConfig, &dataSetWriterIdent) == UA_STATUSCODE_GOOD);
@@ -359,68 +311,11 @@ START_TEST(PublishPDSWithMultipleFieldsAndFixedOffset) {
         ck_assert(UA_Server_freezeWriterGroupConfiguration(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
 
-        UA_ByteString buffer;
-        UA_ByteString_init(&buffer);
-        UA_NetworkMessage networkMessage;
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        ck_assert((*((UA_UInt32 *)networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data)) == 1000);
-        ck_assert(*((UA_UInt32 *) networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields[1].value.data) == 2000);
-        UA_NetworkMessage_clear(&networkMessage);
-        *intValue = (UA_UInt32) 1001;
-        *intValue2 = (UA_UInt32) 2001;
-        UA_ByteString_init(&buffer);
-        memset(&networkMessage, 0, sizeof(networkMessage));
-        ck_assert(UA_Server_setWriterGroupDisabled(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        ck_assert((*((UA_UInt32 *)networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data)) == 1001);
-        ck_assert(*((UA_UInt32 *) networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields[1].value.data) == 2001);
-        UA_NetworkMessage_clear(&networkMessage);
-
-        UA_Server_run_shutdown(server);
-        UA_Server_delete(server);
-        UA_DataValue_delete(dataValue);
-        UA_DataValue_delete(dataValue2);
+        UA_Server_run_iterate(server, false);
 } END_TEST
 
-/* Custom callbacks for add, change and remove pubsub callbacks */
-static UA_StatusCode
-addPubSubApplicationCallback(UA_Server *server_local, UA_NodeId identifier,
-                             UA_ServerCallback callback,
-                             void *data, UA_Double interval_ms,
-                             UA_DateTime *baseTime, UA_TimerPolicy timerPolicy,
-                             UA_UInt64 *callbackId) {
-    UA_StatusCode retval = UA_STATUSCODE_GOOD;
-    UA_WriterGroup *writerGroup = (UA_WriterGroup *)data;
-    /* User defined threads can be called here. For testing, internal repeated callbacks are used */
-    retval = UA_PubSubManager_addRepeatedCallback(server_local,
-                                                  (UA_ServerCallback) callback,
-                                                  writerGroup, writerGroup->config.publishingInterval,
-                                                  baseTime, timerPolicy,
-                                                  &writerGroup->publishCallbackId);
-    return retval;
-}
-
-static UA_StatusCode
-changePubSubApplicationCallback(UA_Server *server_local, UA_NodeId identifier,
-                                UA_UInt64 callbackId, UA_Double interval_ms,
-                                UA_DateTime *baseTime, UA_TimerPolicy timerPolicy) {
-    return UA_STATUSCODE_GOOD;
-}
-
-static void
-removePubSubApplicationCallback(UA_Server *server_local, UA_NodeId identifier, UA_UInt64 callbackId) {
-    /* Using the internal repeated callbacks for the tests. */
-    UA_WriterGroup *writerGroup = UA_WriterGroup_findWGbyId(server_local, identifier);
-    UA_PubSubManager_removeRepeatedPubSubCallback(server_local, writerGroup->publishCallbackId);
-}
-
 START_TEST(PublishSingleFieldInCustomCallback) {
         ck_assert(addMinimalPubSubConfiguration() == UA_STATUSCODE_GOOD);
-        UA_PubSubConnection *connection = UA_PubSubConnection_findConnectionbyId(server, connectionIdentifier);
-        ck_assert(connection);
-        UA_StatusCode rv = connection->channel->regist(connection->channel, NULL, NULL);
-        ck_assert(rv == UA_STATUSCODE_GOOD);
         UA_WriterGroupConfig writerGroupConfig;
         memset(&writerGroupConfig, 0, sizeof(UA_WriterGroupConfig));
         writerGroupConfig.name = UA_STRING("Demo WriterGroup");
@@ -429,9 +324,6 @@ START_TEST(PublishSingleFieldInCustomCallback) {
         writerGroupConfig.writerGroupId = 100;
         writerGroupConfig.encodingMimeType = UA_PUBSUB_ENCODING_UADP;
         writerGroupConfig.rtLevel = UA_PUBSUB_RT_FIXED_SIZE;
-        writerGroupConfig.pubsubManagerCallback.addCustomCallback = addPubSubApplicationCallback;
-        writerGroupConfig.pubsubManagerCallback.changeCustomCallback = changePubSubApplicationCallback;
-        writerGroupConfig.pubsubManagerCallback.removeCustomCallback = removePubSubApplicationCallback;
         UA_UadpWriterGroupMessageDataType *wgm = UA_UadpWriterGroupMessageDataType_new();
         wgm->networkMessageContentMask = UA_UADPNETWORKMESSAGECONTENTMASK_PAYLOADHEADER;
         writerGroupConfig.messageSettings.content.decoded.data = wgm;
@@ -449,10 +341,10 @@ START_TEST(PublishSingleFieldInCustomCallback) {
         // Create Variant and configure as DataSetField source
         UA_UInt32 *intValue = UA_UInt32_new();
         *intValue = (UA_UInt32) 1000;
-        UA_DataValue *dataValue = UA_DataValue_new();
-        UA_Variant_setScalar(&dataValue->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
+        staticSource1 = UA_DataValue_new();
+        UA_Variant_setScalar(&staticSource1->value, intValue, &UA_TYPES[UA_TYPES_UINT32]);
         dsfConfig.field.variable.rtValueSource.rtFieldSourceEnabled = UA_TRUE;
-        dsfConfig.field.variable.rtValueSource.staticValueSource = &dataValue;
+        dsfConfig.field.variable.rtValueSource.staticValueSource = &staticSource1;
         dsfConfig.field.variable.publishParameters.attributeId = UA_ATTRIBUTEID_VALUE;
         ck_assert(UA_Server_addDataSetField(server, publishedDataSetIdent, &dsfConfig, &dataSetFieldIdent).result == UA_STATUSCODE_GOOD);
 
@@ -460,16 +352,8 @@ START_TEST(PublishSingleFieldInCustomCallback) {
 
         ck_assert(UA_Server_freezeWriterGroupConfiguration(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
         ck_assert(UA_Server_setWriterGroupOperational(server, writerGroupIdent) == UA_STATUSCODE_GOOD);
-        UA_ByteString buffer;
-        UA_ByteString_init(&buffer);
-        UA_NetworkMessage networkMessage;
-        receiveSingleMessage(buffer, connection, &networkMessage);
-        ck_assert((*((UA_UInt32 *)networkMessage.payload.dataSetPayload.dataSetMessages->data.keyFrameData.dataSetFields->value.data)) == 1000);
-        UA_NetworkMessage_clear(&networkMessage);
-        UA_DataValue_delete(dataValue);
 
-        UA_Server_run_shutdown(server);
-        UA_Server_delete(server);
+        UA_Server_run_iterate(server, false);
 } END_TEST
 
 static UA_StatusCode
@@ -516,9 +400,9 @@ START_TEST(PubSubConfigWithInformationModelRTVariable) {
         UA_QualifiedName myIntegerName = UA_QUALIFIEDNAME(1, "test node");
         UA_NodeId parentNodeId = UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER);
         UA_NodeId parentReferenceNodeId = UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES);
-        UA_Server_addVariableNode(server, UA_NODEID_NULL, parentNodeId,
+        ck_assert_int_eq(UA_Server_addVariableNode(server, UA_NODEID_NULL, parentNodeId,
                                   parentReferenceNodeId, myIntegerName,
-                                  UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), attr, NULL, &variableNodeId);
+                                  UA_NODEID_NUMERIC(0, UA_NS0ID_BASEDATAVARIABLETYPE), attr, NULL, &variableNodeId), UA_STATUSCODE_GOOD);
         ck_assert(!UA_NodeId_isNull(&variableNodeId));
         UA_Variant variant;
         UA_Variant_init(&variant);
@@ -649,7 +533,7 @@ int main(void) {
     tcase_add_test(tc_pubsub_rt_static_value_source, PubSubConfigWithMultipleInformationModelRTVariables);
 
     TCase *tc_pubsub_rt_fixed_offsets = tcase_create("PubSub RT publish with fixed offsets");
-    tcase_add_checked_fixture(tc_pubsub_rt_fixed_offsets, setup, NULL);
+    tcase_add_checked_fixture(tc_pubsub_rt_fixed_offsets, setup, teardown);
     tcase_add_test(tc_pubsub_rt_fixed_offsets, PublishSingleFieldWithFixedOffsets);
     tcase_add_test(tc_pubsub_rt_fixed_offsets, PublishPDSWithMultipleFieldsAndFixedOffset);
     tcase_add_test(tc_pubsub_rt_fixed_offsets, PublishSingleFieldInCustomCallback);
