@@ -269,9 +269,9 @@ requestHistory(UA_DateTime start,
     request.nodesToReadSize = 1;
     request.nodesToRead = valueId;
 
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     Service_HistoryRead(server, &server->adminSession, &request, response);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     UA_HistoryReadRequest_clear(&request);
 }
 
@@ -468,9 +468,9 @@ deleteHistory(UA_DateTime start,
 
     UA_HistoryUpdateResponse response;
     UA_HistoryUpdateResponse_init(&response);
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     Service_HistoryUpdate(server, &server->adminSession, &request, &response);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     UA_HistoryUpdateRequest_clear(&request);
     UA_StatusCode ret = UA_STATUSCODE_GOOD;
     if (response.responseHeader.serviceResult != UA_STATUSCODE_GOOD)
@@ -522,9 +522,9 @@ updateHistory(UA_PerformUpdateType updateType, UA_DateTime *updateData, UA_Statu
 
     UA_HistoryUpdateResponse response;
     UA_HistoryUpdateResponse_init(&response);
-    UA_LOCK(&server->serviceMutex);
+    lockServer(server);
     Service_HistoryUpdate(server, &server->adminSession, &request, &response);
-    UA_UNLOCK(&server->serviceMutex);
+    unlockServer(server);
     UA_HistoryUpdateRequest_clear(&request);
     UA_StatusCode ret = UA_STATUSCODE_GOOD;
     if (response.responseHeader.serviceResult != UA_STATUSCODE_GOOD)
