@@ -10,7 +10,6 @@
 #include <open62541/types_generated_handling.h>
 
 #include <stdio.h>
-#include <unistd.h>
 
 /* Internal headers */
 #include "ua_pubsub_networkmessage.h"
@@ -238,7 +237,7 @@ int main(int argc, char **argv) {
     /* Read input until EOF */
     size_t pos = 0;
     size_t length = 128;
-    while(true) {
+    do {
         if(pos >= buf.length) {
             length = length * 8;
             UA_Byte *r = (UA_Byte*)realloc(buf.data, length);
@@ -250,16 +249,14 @@ int main(int argc, char **argv) {
             buf.data = r;
         }
 
-        ssize_t c = read(fileno(in), &buf.data[pos], length - pos);
-        if(c == 0)
-            break;
-        if(c < 0) {
+        size_t c = fread(&buf.data[pos], sizeof(UA_Byte), length - pos, in);
+        if(ferror(in)) {
             fprintf(stderr, "Reading from input failed\n");
             goto cleanup;
         }
 
-        pos += (size_t)c;
-    }
+        pos += c;
+    } while (!feof(in));
 
     if(pos == 0) {
         fprintf(stderr, "No input\n");
